@@ -249,7 +249,7 @@ ConfigOption* ConfigOptionDef::create_default_option() const
             // Special case: For a DynamicConfig, convert a templated enum to a generic enum.
             new ConfigOptionEnumGeneric(this->enum_keys_map, this->default_value->getInt()) :
             this->default_value->clone();
-	return this->create_empty_option();
+    return this->create_empty_option();
 }
 
 // Assignment of the serialization IDs is not thread safe. The Defs shall be initialized from the main thread!
@@ -271,8 +271,6 @@ ConfigOptionDef* ConfigDef::add_nullable(const t_config_option_key &opt_key, Con
 	return def;
 }
 
-std::string ConfigOptionDef::nocli = "~~~noCLI";
-
 std::ostream& ConfigDef::print_cli_help(std::ostream& out, bool show_defaults, std::function<bool(const ConfigOptionDef &)> filter) const
 {
     // prepare a function for wrapping text
@@ -427,7 +425,30 @@ std::string ConfigBase::opt_serialize(const t_config_option_key &opt_key) const
     return opt->serialize();
 }
 
-bool ConfigBase::set_deserialize(const t_config_option_key &opt_key_src, const std::string &value_src, bool append)
+void ConfigBase::set(const std::string &opt_key, int value, bool create)
+{
+    ConfigOption *opt = this->option_throw(opt_key, create);
+    switch (opt->type()) {
+    	case coInt:    static_cast<ConfigOptionInt*>(opt)->value = value; break;
+    	case coFloat:  static_cast<ConfigOptionFloat*>(opt)->value = value; break;
+		case coFloatOrPercent:  static_cast<ConfigOptionFloatOrPercent*>(opt)->value = value; static_cast<ConfigOptionFloatOrPercent*>(opt)->percent = false; break;
+		case coString: static_cast<ConfigOptionString*>(opt)->value = std::to_string(value); break;
+    	default: throw BadOptionTypeException("Configbase::set() - conversion from int not possible");
+    }
+}
+
+void ConfigBase::set(const std::string &opt_key, double value, bool create)
+{
+    ConfigOption *opt = this->option_throw(opt_key, create);
+    switch (opt->type()) {
+    	case coFloat:  			static_cast<ConfigOptionFloat*>(opt)->value = value; break;
+    	case coFloatOrPercent:  static_cast<ConfigOptionFloatOrPercent*>(opt)->value = value; static_cast<ConfigOptionFloatOrPercent*>(opt)->percent = false; break;
+		case coString: 			static_cast<ConfigOptionString*>(opt)->value = std::to_string(value); break;
+    	default: throw BadOptionTypeException("Configbase::set() - conversion from float not possible");
+    }
+}
+
+bool ConfigBase::set_deserialize_nothrow(const t_config_option_key &opt_key_src, const std::string &value_src, bool append)
 {
     t_config_option_key opt_key = opt_key_src;
     std::string         value   = value_src;
@@ -440,6 +461,18 @@ bool ConfigBase::set_deserialize(const t_config_option_key &opt_key_src, const s
     return this->set_deserialize_raw(opt_key, value, append);
 }
 
+void ConfigBase::set_deserialize(const t_config_option_key &opt_key_src, const std::string &value_src, bool append)
+{
+	if (! this->set_deserialize_nothrow(opt_key_src, value_src, append))
+		throw BadOptionTypeException("ConfigBase::set_deserialize() failed");
+}
+
+void ConfigBase::set_deserialize(std::initializer_list<SetDeserializeItem> items)
+{
+	for (const SetDeserializeItem &item : items)
+		this->set_deserialize(item.opt_key, item.opt_value, item.append);
+}
+
 bool ConfigBase::set_deserialize_raw(const t_config_option_key &opt_key_src, const std::string &value, bool append)
 {
     t_config_option_key opt_key = opt_key_src;
@@ -579,7 +612,7 @@ void ConfigBase::load_from_gcode_file(const std::string &file)
 	}
     ifs.seekg(0, ifs.end);
 	auto file_length = ifs.tellg();
-	auto data_length = std::min<std::fstream::streampos>(65535, file_length);
+	auto data_length = std::min<std::fstream::pos_type>(65535, file_length);
 	ifs.seekg(file_length - data_length, ifs.beg);
     std::vector<char> data(size_t(data_length) + 1, 0);
     ifs.read(data.data(), data_length);
@@ -597,39 +630,38 @@ size_t ConfigBase::load_from_gcode_string(const char* str)
         return 0;
 
     // Walk line by line in reverse until a non-configuration key appears.
-    char *data_start = const_cast<char*>(str);
+    const char *data_start = str;
     // boost::nowide::ifstream seems to cook the text data somehow, so less then the 64k of characters may be retrieved.
-    char *end = data_start + strlen(str);
+    const char *end = data_start + strlen(str);
     size_t num_key_value_pairs = 0;
     for (;;) {
         // Extract next line.
         for (--end; end > data_start && (*end == '\r' || *end == '\n'); --end);
         if (end == data_start)
             break;
-        char *start = end;
-        *(++end) = 0;
+        const char *start = end ++;
         for (; start > data_start && *start != '\r' && *start != '\n'; --start);
         if (start == data_start)
             break;
         // Extracted a line from start to end. Extract the key = value pair.
-        if (end - (++start) < 10 || start[0] != ';' || start[1] != ' ')
+        if (end - (++ start) < 10 || start[0] != ';' || start[1] != ' ')
             break;
-        char *key = start + 2;
+        const char *key = start + 2;
         if (!(*key >= 'a' && *key <= 'z') || (*key >= 'A' && *key <= 'Z'))
             // A key must start with a letter.
             break;
-        char *sep = strchr(key, '=');
-        if (sep == nullptr || sep[-1] != ' ' || sep[1] != ' ')
+        const char *sep = key;
+        for (; sep != end && *sep != '='; ++ sep) ;
+        if (sep == end || sep[-1] != ' ' || sep[1] != ' ')
             break;
-        char *value = sep + 2;
+        const char *value = sep + 2;
         if (value > end)
             break;
-        char *key_end = sep - 1;
+        const char *key_end = sep - 1;
         if (key_end - key < 3)
             break;
-        *key_end = 0;
         // The key may contain letters, digits and underscores.
-        for (char *c = key; c != key_end; ++c)
+        for (const char *c = key; c != key_end; ++ c)
             if (!((*c >= 'a' && *c <= 'z') || (*c >= 'A' && *c <= 'Z') || (*c >= '0' && *c <= '9') || *c == '_')) {
                 key = nullptr;
                 break;
@@ -637,7 +669,7 @@ size_t ConfigBase::load_from_gcode_string(const char* str)
         if (key == nullptr)
             break;
         try {
-            this->set_deserialize(key, value);
+            this->set_deserialize(std::string(key, key_end), std::string(value, end));
             ++num_key_value_pairs;
         }
         catch (UnknownOptionException & /* e */) {
@@ -670,6 +702,12 @@ void ConfigBase::null_nullables()
     }
 }
 
+DynamicConfig::DynamicConfig(const ConfigBase& rhs, const t_config_option_keys& keys)
+{
+	for (const t_config_option_key& opt_key : keys)
+		this->options[opt_key] = std::unique_ptr<ConfigOption>(rhs.option(opt_key)->clone());
+}
+
 bool DynamicConfig::operator==(const DynamicConfig &rhs) const
 {
     auto it1     = this->options.begin();
@@ -719,17 +757,23 @@ ConfigOption* DynamicConfig::optptr(const t_config_option_key &opt_key, bool cre
     return opt;
 }
 
-void DynamicConfig::read_cli(const std::vector<std::string> &tokens, t_config_option_keys* extra, t_config_option_keys* keys)
+const ConfigOption* DynamicConfig::optptr(const t_config_option_key &opt_key) const
 {
-    std::vector<char*> args;    
-    // push a bogus executable name (argv[0])
-    args.emplace_back(const_cast<char*>(""));
-    for (size_t i = 0; i < tokens.size(); ++ i)
-        args.emplace_back(const_cast<char *>(tokens[i].c_str()));
-    this->read_cli(int(args.size()), &args[0], extra, keys);
+    auto it = options.find(opt_key);
+    return (it == options.end()) ? nullptr : it->second.get();
 }
 
-bool DynamicConfig::read_cli(int argc, char** argv, t_config_option_keys* extra, t_config_option_keys* keys)
+void DynamicConfig::read_cli(const std::vector<std::string> &tokens, t_config_option_keys* extra, t_config_option_keys* keys)
+{
+    std::vector<const char*> args;    
+    // push a bogus executable name (argv[0])
+    args.emplace_back("");
+    for (size_t i = 0; i < tokens.size(); ++ i)
+        args.emplace_back(tokens[i].c_str());
+    this->read_cli(int(args.size()), args.data(), extra, keys);
+}
+
+bool DynamicConfig::read_cli(int argc, const char* const argv[], t_config_option_keys* extra, t_config_option_keys* keys)
 {
     // cache the CLI option => opt_key mapping
     std::map<std::string,std::string> opts;
@@ -819,7 +863,7 @@ bool DynamicConfig::read_cli(int argc, char** argv, t_config_option_keys* extra,
             static_cast<ConfigOptionString*>(opt_base)->value = value;
         } else {
             // Any scalar value of a type different from Bool and String.
-            if (! this->set_deserialize(opt_key, value, false)) {
+            if (! this->set_deserialize_nothrow(opt_key, value, false)) {
 				boost::nowide::cerr << "Invalid value supplied for --" << token.c_str() << std::endl;
 				return false;
 			}
