@@ -16,6 +16,17 @@ public:
     // require bridge flow since most of this pattern hangs in air
     virtual bool use_bridge_flow() const { return false; }
 
+    // Correction applied to regular infill angle to maximize printing
+    // speed in default configuration (degrees)
+    static constexpr float CorrectionAngle = -45.;
+
+    // Density adjustment to have a good %of weight.
+    static constexpr double DensityAdjust = 2.44;
+
+    // Gyroid upper resolution tolerance (mm^-2)
+    static constexpr double PatternTolerance = 0.2;
+
+
 protected:
     virtual void _fill_surface_single(
         const FillParams                &params, 
