@@ -20,6 +20,7 @@ static const unsigned int DEFAULT_EXTRUDER_ID = 0;
 static const unsigned int DEFAULT_COLOR_PRINT_ID = 0;
 static const Slic3r::Vec3d DEFAULT_START_POSITION = Slic3r::Vec3d(0.0f, 0.0f, 0.0f);
 static const float DEFAULT_START_EXTRUSION = 0.0f;
+static const float DEFAULT_FAN_SPEED = 0.0f;
 
 namespace Slic3r {
 
@@ -28,6 +29,9 @@ const std::string GCodeAnalyzer::Mm3_Per_Mm_Tag = "_ANALYZER_MM3_PER_MM:";
 const std::string GCodeAnalyzer::Width_Tag = "_ANALYZER_WIDTH:";
 const std::string GCodeAnalyzer::Height_Tag = "_ANALYZER_HEIGHT:";
 const std::string GCodeAnalyzer::Color_Change_Tag = "_ANALYZER_COLOR_CHANGE";
+const std::string GCodeAnalyzer::Pause_Print_Tag = "_ANALYZER_PAUSE_PRINT";
+const std::string GCodeAnalyzer::Custom_Code_Tag = "_ANALYZER_CUSTOM_CODE";
+const std::string GCodeAnalyzer::End_Pause_Print_Or_Custom_Code_Tag = "_ANALYZER_END_PAUSE_PRINT_OR_CUSTOM_CODE";
 
 const double GCodeAnalyzer::Default_mm3_per_mm = 0.0;
 const float GCodeAnalyzer::Default_Width = 0.0f;
@@ -36,21 +40,23 @@ const float GCodeAnalyzer::Default_Height = 0.0f;
 GCodeAnalyzer::Metadata::Metadata()
     : extrusion_role(erNone)
     , extruder_id(DEFAULT_EXTRUDER_ID)
-    , cp_color_id(DEFAULT_COLOR_PRINT_ID)
     , mm3_per_mm(GCodeAnalyzer::Default_mm3_per_mm)
     , width(GCodeAnalyzer::Default_Width)
     , height(GCodeAnalyzer::Default_Height)
     , feedrate(DEFAULT_FEEDRATE)
+    , fan_speed(DEFAULT_FAN_SPEED)
+    , cp_color_id(DEFAULT_COLOR_PRINT_ID)
 {
 }
 
-GCodeAnalyzer::Metadata::Metadata(ExtrusionRole extrusion_role, unsigned int extruder_id, double mm3_per_mm, float width, float height, float feedrate, unsigned int cp_color_id/* = 0*/)
+GCodeAnalyzer::Metadata::Metadata(ExtrusionRole extrusion_role, unsigned int extruder_id, double mm3_per_mm, float width, float height, float feedrate, float fan_speed, unsigned int cp_color_id/* = 0*/)
     : extrusion_role(extrusion_role)
     , extruder_id(extruder_id)
     , mm3_per_mm(mm3_per_mm)
     , width(width)
     , height(height)
     , feedrate(feedrate)
+    , fan_speed(fan_speed)
     , cp_color_id(cp_color_id)
 {
 }
@@ -75,15 +81,18 @@ bool GCodeAnalyzer::Metadata::operator != (const GCodeAnalyzer::Metadata& other)
     if (feedrate != other.feedrate)
         return true;
 
+    if (fan_speed != other.fan_speed)
+        return true;
+
     if (cp_color_id != other.cp_color_id)
         return true;
 
     return false;
 }
 
-GCodeAnalyzer::GCodeMove::GCodeMove(GCodeMove::EType type, ExtrusionRole extrusion_role, unsigned int extruder_id, double mm3_per_mm, float width, float height, float feedrate, const Vec3d& start_position, const Vec3d& end_position, float delta_extruder, unsigned int cp_color_id/* = 0*/)
+GCodeAnalyzer::GCodeMove::GCodeMove(GCodeMove::EType type, ExtrusionRole extrusion_role, unsigned int extruder_id, double mm3_per_mm, float width, float height, float feedrate, const Vec3d& start_position, const Vec3d& end_position, float delta_extruder, float fan_speed, unsigned int cp_color_id/* = 0*/)
     : type(type)
-    , data(extrusion_role, extruder_id, mm3_per_mm, width, height, feedrate, cp_color_id)
+    , data(extrusion_role, extruder_id, mm3_per_mm, width, height, feedrate, fan_speed, cp_color_id)
     , start_position(start_position)
     , end_position(end_position)
     , delta_extruder(delta_extruder)
@@ -99,24 +108,11 @@ GCodeAnalyzer::GCodeMove::GCodeMove(GCodeMove::EType type, const GCodeAnalyzer::
 {
 }
 
-GCodeAnalyzer::GCodeAnalyzer()
-{
-    reset();
-}
-
-void GCodeAnalyzer::set_extruder_offsets(const GCodeAnalyzer::ExtruderOffsetsMap& extruder_offsets)
-{
-    m_extruder_offsets = extruder_offsets;
-}
-
 void GCodeAnalyzer::set_extruders_count(unsigned int count)
 {
     m_extruders_count = count;
-}
-
-void GCodeAnalyzer::set_gcode_flavor(const GCodeFlavor& flavor)
-{
-    m_gcode_flavor = flavor;
+    for (unsigned int i=0; i<m_extruders_count; i++)
+        m_extruder_color[i] = i;
 }
 
 void GCodeAnalyzer::reset()
@@ -133,12 +129,15 @@ void GCodeAnalyzer::reset()
     _set_feedrate(DEFAULT_FEEDRATE);
     _set_start_position(DEFAULT_START_POSITION);
     _set_start_extrusion(DEFAULT_START_EXTRUSION);
+    _set_fan_speed(DEFAULT_FAN_SPEED);
     _reset_axes_position();
+    _reset_axes_origin();
     _reset_cached_position();
 
     m_moves_map.clear();
     m_extruder_offsets.clear();
     m_extruders_count = 1;
+    m_extruder_color.clear();
 }
 
 const std::string& GCodeAnalyzer::process_gcode(const std::string& gcode)
@@ -259,6 +258,16 @@ void GCodeAnalyzer::_process_gcode_line(GCodeReader&, const GCodeReader::GCodeLi
                         _processM83(line);
                         break;
                     }
+                case 106: // Set fan speed
+                    {
+                        _processM106(line);
+                        break;
+                    }
+                case 107: // Disable fan
+                    {
+                        _processM107(line);
+                        break;
+                    }
                 case 108:
                 case 135:
                     {
@@ -267,6 +276,11 @@ void GCodeAnalyzer::_process_gcode_line(GCodeReader&, const GCodeReader::GCodeLi
                         _processM108orM135(line);
                         break;
                     }
+                case 132: // Recall stored home offsets
+                    {
+                        _processM132(line);
+                        break;
+                    }
                 case 401: // Repetier: Store x, y and z position
                     {
                         _processM401(line);
@@ -293,31 +307,30 @@ void GCodeAnalyzer::_process_gcode_line(GCodeReader&, const GCodeReader::GCodeLi
     m_process_output += line.raw() + "\n";
 }
 
-// Returns the new absolute position on the given axis in dependence of the given parameters
-float axis_absolute_position_from_G1_line(GCodeAnalyzer::EAxis axis, const GCodeReader::GCodeLine& lineG1, GCodeAnalyzer::EUnits units, bool is_relative, float current_absolute_position)
-{
-    float lengthsScaleFactor = (units == GCodeAnalyzer::Inches) ? INCHES_TO_MM : 1.0f;
-    if (lineG1.has(Slic3r::Axis(axis)))
-    {
-        float ret = lineG1.value(Slic3r::Axis(axis)) * lengthsScaleFactor;
-        return is_relative ? current_absolute_position + ret : ret;
-    }
-    else
-        return current_absolute_position;
-}
-
 void GCodeAnalyzer::_processG1(const GCodeReader::GCodeLine& line)
 {
+    auto axis_absolute_position = [this](GCodeAnalyzer::EAxis axis, const GCodeReader::GCodeLine& lineG1) -> float
+    {
+        bool is_relative = (_get_global_positioning_type() == Relative);
+        if (axis == E)
+            is_relative |= (_get_e_local_positioning_type() == Relative);
+
+        if (lineG1.has(Slic3r::Axis(axis)))
+        {
+            float lengthsScaleFactor = (_get_units() == GCodeAnalyzer::Inches) ? INCHES_TO_MM : 1.0f;
+            float ret = lineG1.value(Slic3r::Axis(axis)) * lengthsScaleFactor;
+            return is_relative ? _get_axis_position(axis) + ret : _get_axis_origin(axis) + ret;
+        }
+        else
+            return _get_axis_position(axis);
+    };
+
     // updates axes positions from line
-    EUnits units = _get_units();
+
     float new_pos[Num_Axis];
     for (unsigned char a = X; a < Num_Axis; ++a)
     {
-        bool is_relative = (_get_global_positioning_type() == Relative);
-        if (a == E)
-            is_relative |= (_get_e_local_positioning_type() == Relative);
-
-        new_pos[a] = axis_absolute_position_from_G1_line((EAxis)a, line, units, is_relative, _get_axis_position((EAxis)a));
+        new_pos[a] = axis_absolute_position((EAxis)a, line);
     }
 
     // updates feedrate from line, if present
@@ -337,7 +350,7 @@ void GCodeAnalyzer::_processG1(const GCodeReader::GCodeLine& line)
     if (delta_pos[E] < 0.0f)
     {
         if ((delta_pos[X] != 0.0f) || (delta_pos[Y] != 0.0f) || (delta_pos[Z] != 0.0f))
-            type = GCodeMove::Move;
+        type = GCodeMove::Move;
         else
             type = GCodeMove::Retract;
     }
@@ -407,33 +420,37 @@ void GCodeAnalyzer::_processG92(const GCodeReader::GCodeLine& line)
 
     if (line.has_x())
     {
-        _set_axis_position(X, line.x() * lengthsScaleFactor);
+        _set_axis_origin(X, _get_axis_position(X) - line.x() * lengthsScaleFactor);
         anyFound = true;
     }
 
     if (line.has_y())
     {
-        _set_axis_position(Y, line.y() * lengthsScaleFactor);
+        _set_axis_origin(Y, _get_axis_position(Y) - line.y() * lengthsScaleFactor);
         anyFound = true;
     }
 
     if (line.has_z())
     {
-        _set_axis_position(Z, line.z() * lengthsScaleFactor);
+        _set_axis_origin(Z, _get_axis_position(Z) - line.z() * lengthsScaleFactor);
         anyFound = true;
     }
 
     if (line.has_e())
     {
+        // extruder coordinate can grow to the point where its float representation does not allow for proper addition with small increments,
+        // we set the value taken from the G92 line as the new current position for it
         _set_axis_position(E, line.e() * lengthsScaleFactor);
         anyFound = true;
     }
 
-    if (!anyFound)
+    if (!anyFound && ! line.has_unknown_axis())
     {
+    	// The G92 may be called for axes that PrusaSlicer does not recognize, for example see GH issue #3510, 
+    	// where G92 A0 B0 is called although the extruder axis is till E.
         for (unsigned char a = X; a < Num_Axis; ++a)
         {
-            _set_axis_position((EAxis)a, 0.0f);
+            _set_axis_origin((EAxis)a, _get_axis_position((EAxis)a));
         }
     }
 }
@@ -448,6 +465,24 @@ void GCodeAnalyzer::_processM83(const GCodeReader::GCodeLine& line)
     _set_e_local_positioning_type(Relative);
 }
 
+void GCodeAnalyzer::_processM106(const GCodeReader::GCodeLine& line)
+{
+    if (!line.has('P'))
+    {
+        // The absence of P means the print cooling fan, so ignore anything else.
+        float new_fan_speed;
+        if (line.has_value('S', new_fan_speed))
+            _set_fan_speed((100.0f / 255.0f) * new_fan_speed);
+        else
+            _set_fan_speed(100.0f);
+    }
+}
+
+void GCodeAnalyzer::_processM107(const GCodeReader::GCodeLine& line)
+{
+    _set_fan_speed(0.0f);
+}
+
 void GCodeAnalyzer::_processM108orM135(const GCodeReader::GCodeLine& line)
 {
     // These M-codes are used by MakerWare and Sailfish to change active tool.
@@ -467,6 +502,25 @@ void GCodeAnalyzer::_processM108orM135(const GCodeReader::GCodeLine& line)
     }
 }
 
+void GCodeAnalyzer::_processM132(const GCodeReader::GCodeLine& line)
+{
+    // This command is used by Makerbot to load the current home position from EEPROM
+    // see: https://github.com/makerbot/s3g/blob/master/doc/GCodeProtocol.md
+    // Using this command to reset the axis origin to zero helps in fixing: https://github.com/prusa3d/PrusaSlicer/issues/3082
+
+    if (line.has_x())
+        _set_axis_origin(X, 0.0f);
+
+    if (line.has_y())
+        _set_axis_origin(Y, 0.0f);
+
+    if (line.has_z())
+        _set_axis_origin(Z, 0.0f);
+
+    if (line.has_e())
+        _set_axis_origin(E, 0.0f);
+}
+
 void GCodeAnalyzer::_processM401(const GCodeReader::GCodeLine& line)
 {
     if (m_gcode_flavor != gcfRepetier)
@@ -534,7 +588,11 @@ void GCodeAnalyzer::_processT(const std::string& cmd)
                     BOOST_LOG_TRIVIAL(error) << "GCodeAnalyzer encountered an invalid toolchange, maybe from a custom gcode.";
             }
             else
+            {
                 _set_extruder_id(id);
+                if (_get_cp_color_id() != INT_MAX)
+                    _set_cp_color_id(m_extruder_color[id]);
+            }
 
             // stores tool change move
             _store_move(GCodeMove::Tool_change);
@@ -587,7 +645,33 @@ bool GCodeAnalyzer::_process_tags(const GCodeReader::GCodeLine& line)
     pos = comment.find(Color_Change_Tag);
     if (pos != comment.npos)
     {
-        _process_color_change_tag();
+        pos = comment.find_last_of(",T");
+        int extruder = pos == comment.npos ? 0 : std::atoi(comment.substr(pos + 1, comment.npos).c_str());
+        _process_color_change_tag(extruder);
+        return true;
+    }
+
+    // color change tag
+    pos = comment.find(Pause_Print_Tag);
+    if (pos != comment.npos)
+    {
+        _process_pause_print_or_custom_code_tag();
+        return true;
+    }
+
+    // color change tag
+    pos = comment.find(Custom_Code_Tag);
+    if (pos != comment.npos)
+    {
+        _process_pause_print_or_custom_code_tag();
+        return true;
+    }
+
+    // color change tag
+    pos = comment.find(End_Pause_Print_Or_Custom_Code_Tag);
+    if (pos != comment.npos)
+    {
+        _process_end_pause_print_or_custom_code_tag();
         return true;
     }
 
@@ -620,10 +704,24 @@ void GCodeAnalyzer::_process_height_tag(const std::string& comment, size_t pos)
     _set_height((float)::strtod(comment.substr(pos + Height_Tag.length()).c_str(), nullptr));
 }
 
-void GCodeAnalyzer::_process_color_change_tag()
+void GCodeAnalyzer::_process_color_change_tag(int extruder)
 {
-    m_state.cur_cp_color_id++;
-    _set_cp_color_id(m_state.cur_cp_color_id);
+    m_extruder_color[extruder] = m_extruders_count + m_state.cp_color_counter; // color_change position in list of color for preview
+    m_state.cp_color_counter++;
+
+    if (_get_extruder_id() == extruder)
+        _set_cp_color_id(m_extruder_color[extruder]);
+}
+
+void GCodeAnalyzer::_process_pause_print_or_custom_code_tag()
+{
+    _set_cp_color_id(INT_MAX);
+}
+
+void GCodeAnalyzer::_process_end_pause_print_or_custom_code_tag()
+{
+    if (_get_cp_color_id() == INT_MAX)
+        _set_cp_color_id(m_extruder_color[_get_extruder_id()]);
 }
 
 void GCodeAnalyzer::_set_units(GCodeAnalyzer::EUnits units)
@@ -726,6 +824,16 @@ float GCodeAnalyzer::_get_feedrate() const
     return m_state.data.feedrate;
 }
 
+void GCodeAnalyzer::_set_fan_speed(float fan_speed_percentage)
+{
+    m_state.data.fan_speed = fan_speed_percentage;
+}
+
+float GCodeAnalyzer::_get_fan_speed() const
+{
+    return m_state.data.fan_speed;
+}
+
 void GCodeAnalyzer::_set_axis_position(EAxis axis, float position)
 {
     m_state.position[axis] = position;
@@ -736,11 +844,26 @@ float GCodeAnalyzer::_get_axis_position(EAxis axis) const
     return m_state.position[axis];
 }
 
+void GCodeAnalyzer::_set_axis_origin(EAxis axis, float position)
+{
+    m_state.origin[axis] = position;
+}
+
+float GCodeAnalyzer::_get_axis_origin(EAxis axis) const
+{
+    return m_state.origin[axis];
+}
+
 void GCodeAnalyzer::_reset_axes_position()
 {
     ::memset((void*)m_state.position, 0, Num_Axis * sizeof(float));
 }
 
+void GCodeAnalyzer::_reset_axes_origin()
+{
+    ::memset((void*)m_state.origin, 0, Num_Axis * sizeof(float));
+}
+
 void GCodeAnalyzer::_set_start_position(const Vec3d& position)
 {
     m_state.start_position = position;
@@ -798,7 +921,7 @@ void GCodeAnalyzer::_store_move(GCodeAnalyzer::GCodeMove::EType type)
 
     Vec3d start_position = _get_start_position() + extruder_offset;
     Vec3d end_position = _get_end_position() + extruder_offset;
-    it->second.emplace_back(type, _get_extrusion_role(), extruder_id, _get_mm3_per_mm(), _get_width(), _get_height(), _get_feedrate(), start_position, end_position, _get_delta_extrusion(), _get_cp_color_id());
+    it->second.emplace_back(type, _get_extrusion_role(), extruder_id, _get_mm3_per_mm(), _get_width(), _get_height(), _get_feedrate(), start_position, end_position, _get_delta_extrusion(), _get_fan_speed(), _get_cp_color_id());
 }
 
 bool GCodeAnalyzer::_is_valid_extrusion_role(int value) const
@@ -821,7 +944,7 @@ void GCodeAnalyzer::_calc_gcode_preview_extrusion_layers(GCodePreviewData& previ
             }
 
             // if layer not found, create and return it
-            layers.emplace_back(z, ExtrusionPaths());
+            layers.emplace_back(z, GCodePreviewData::Extrusion::Paths());
             return layers.back();
         }
 
@@ -830,13 +953,18 @@ void GCodeAnalyzer::_calc_gcode_preview_extrusion_layers(GCodePreviewData& previ
             // if the polyline is valid, create the extrusion path from it and store it
             if (polyline.is_valid())
             {
-                ExtrusionPath path(data.extrusion_role, data.mm3_per_mm, data.width, data.height);
+				auto& paths = get_layer_at_z(preview_data.extrusion.layers, z).paths;
+				paths.emplace_back(GCodePreviewData::Extrusion::Path());
+				GCodePreviewData::Extrusion::Path &path = paths.back();
                 path.polyline = polyline;
+				path.extrusion_role = data.extrusion_role;
+				path.mm3_per_mm = float(data.mm3_per_mm);
+				path.width = data.width;
+				path.height = data.height;
                 path.feedrate = data.feedrate;
                 path.extruder_id = data.extruder_id;
                 path.cp_color_id = data.cp_color_id;
-
-                get_layer_at_z(preview_data.extrusion.layers, z).paths.push_back(path);
+                path.fan_speed = data.fan_speed;
             }
         }
     };
@@ -852,8 +980,9 @@ void GCodeAnalyzer::_calc_gcode_preview_extrusion_layers(GCodePreviewData& previ
     float volumetric_rate = FLT_MAX;
     GCodePreviewData::Range height_range;
     GCodePreviewData::Range width_range;
-    GCodePreviewData::Range feedrate_range;
+    GCodePreviewData::MultiRange<GCodePreviewData::FeedrateKind> feedrate_range;
     GCodePreviewData::Range volumetric_rate_range;
+    GCodePreviewData::Range fan_speed_range;
 
     // to avoid to call the callback too often
     unsigned int cancel_callback_threshold = (unsigned int)std::max((int)extrude_moves->second.size() / 25, 1);
@@ -886,8 +1015,9 @@ void GCodeAnalyzer::_calc_gcode_preview_extrusion_layers(GCodePreviewData& previ
             volumetric_rate = move.data.feedrate * (float)move.data.mm3_per_mm;
             height_range.update_from(move.data.height);
             width_range.update_from(move.data.width);
-            feedrate_range.update_from(move.data.feedrate);
+            feedrate_range.update_from(move.data.feedrate, GCodePreviewData::FeedrateKind::EXTRUSION);
             volumetric_rate_range.update_from(volumetric_rate);
+            fan_speed_range.update_from(move.data.fan_speed);
         }
         else
             // append end vertex of the move to current polyline
@@ -906,6 +1036,7 @@ void GCodeAnalyzer::_calc_gcode_preview_extrusion_layers(GCodePreviewData& previ
     preview_data.ranges.width.update_from(width_range);
     preview_data.ranges.feedrate.update_from(feedrate_range);
     preview_data.ranges.volumetric_rate.update_from(volumetric_rate_range);
+    preview_data.ranges.fan_speed.update_from(fan_speed_range);
 
     // we need to sort the layers by their z as they can be shuffled in case of sequential prints
     std::sort(preview_data.extrusion.layers.begin(), preview_data.extrusion.layers.end(), [](const GCodePreviewData::Extrusion::Layer& l1, const GCodePreviewData::Extrusion::Layer& l2)->bool { return l1.z < l2.z; });
@@ -937,7 +1068,7 @@ void GCodeAnalyzer::_calc_gcode_preview_travel(GCodePreviewData& preview_data, s
 
     GCodePreviewData::Range height_range;
     GCodePreviewData::Range width_range;
-    GCodePreviewData::Range feedrate_range;
+    GCodePreviewData::MultiRange<GCodePreviewData::FeedrateKind> feedrate_range;
 
     // to avoid to call the callback too often
     unsigned int cancel_callback_threshold = (unsigned int)std::max((int)travel_moves->second.size() / 25, 1);
@@ -977,7 +1108,7 @@ void GCodeAnalyzer::_calc_gcode_preview_travel(GCodePreviewData& preview_data, s
         extruder_id = move.data.extruder_id;
         height_range.update_from(move.data.height);
         width_range.update_from(move.data.width);
-        feedrate_range.update_from(move.data.feedrate);
+        feedrate_range.update_from(move.data.feedrate, GCodePreviewData::FeedrateKind::TRAVEL);
     }
 
     // store last polyline
