@@ -69,7 +69,7 @@ public:
     }
 
     WipeTowerWriter&          change_analyzer_mm3_per_mm(float len, float e) {
-            static const float area = M_PI * 1.75f * 1.75f / 4.f;
+            static const float area = float(M_PI) * 1.75f * 1.75f / 4.f;
             float mm3_per_mm = (len == 0.f ? 0.f : area * e / len);
             // adds tag for analyzer:
             char buf[64];
@@ -87,7 +87,7 @@ public:
 		return *this;
 	}
 
-	WipeTowerWriter&				 set_initial_tool(const unsigned int tool) { m_current_tool = tool; return *this; }
+    WipeTowerWriter&				 set_initial_tool(size_t tool) { m_current_tool = tool; return *this; }
 
 	WipeTowerWriter&				 set_z(float z) 
 		{ m_current_z = z; return *this; }
@@ -138,7 +138,7 @@ public:
 
 		float dx = x - m_current_pos.x();
 		float dy = y - m_current_pos.y();
-		double len = sqrt(dx*dx+dy*dy);
+        float len = std::sqrt(dx*dx+dy*dy);
         if (record_length)
             m_used_filament_length += e;
 
@@ -146,11 +146,11 @@ public:
 		Vec2f rotated_current_pos(this->pos_rotated());
 		Vec2f rot(this->rotate(Vec2f(x,y)));                               // this is where we want to go
 
-		if (! m_preview_suppressed && e > 0.f && len > 0.) {
+        if (! m_preview_suppressed && e > 0.f && len > 0.f) {
             change_analyzer_mm3_per_mm(len, e);
 			// Width of a squished extrusion, corrected for the roundings of the squished extrusions.
 			// This is left zero if it is a travel move.
-			float width = float(double(e) * m_filpar[0].filament_area / (len * m_layer_height));
+            float width = e * m_filpar[0].filament_area / (len * m_layer_height);
 			// Correct for the roundings of a squished extrusion.
 			width += m_layer_height * float(1. - M_PI / 4.);
 			if (m_extrusions.empty() || m_extrusions.back().pos != rotated_current_pos)
@@ -159,10 +159,10 @@ public:
 		}
 
 		m_gcode += "G1";
-		if (std::abs(rot.x() - rotated_current_pos.x()) > EPSILON)
+        if (std::abs(rot.x() - rotated_current_pos.x()) > (float)EPSILON)
 			m_gcode += set_format_X(rot.x());
 
-		if (std::abs(rot.y() - rotated_current_pos.y()) > EPSILON)
+        if (std::abs(rot.y() - rotated_current_pos.y()) > (float)EPSILON)
 			m_gcode += set_format_Y(rot.y());
 
 
@@ -171,7 +171,7 @@ public:
 
 		if (f != 0.f && f != m_current_feedrate) {
             if (limit_volumetric_flow) {
-                float e_speed = e / (((len == 0) ? std::abs(e) : len) / f * 60.f);
+                float e_speed = e / (((len == 0.f) ? std::abs(e) : len) / f * 60.f);
                 f /= std::max(1.f, e_speed / m_filpar[m_current_tool].max_e_speed);
             }
 			m_gcode += set_format_F(f);
@@ -181,7 +181,7 @@ public:
         m_current_pos.y() = y;
 
 		// Update the elapsed time with a rough estimate.
-		m_elapsed_time += ((len == 0) ? std::abs(e) : len) / m_current_feedrate * 60.f;
+        m_elapsed_time += ((len == 0.f) ? std::abs(e) : len) / m_current_feedrate * 60.f;
 		m_gcode += "\n";
 		return *this;
 	}
@@ -201,7 +201,7 @@ public:
 	{
 		float dx = x - m_current_pos.x();
 		float dy = y - m_current_pos.y();
-		return extrude_explicit(x, y, sqrt(dx*dx+dy*dy) * m_extrusion_flow, f, true);
+        return extrude_explicit(x, y, std::sqrt(dx*dx+dy*dy) * m_extrusion_flow, f, true);
 	}
 
 	WipeTowerWriter& extrude(const Vec2f &dest, const float f = 0.f) 
@@ -298,7 +298,7 @@ public:
 		return *this;
 	}
 
-	WipeTowerWriter& set_tool(int tool)
+    WipeTowerWriter& set_tool(size_t tool)
 	{
 		m_current_tool = tool;
 		return *this;
@@ -307,57 +307,55 @@ public:
 	// Set extruder temperature, don't wait by default.
 	WipeTowerWriter& set_extruder_temp(int temperature, bool wait = false)
 	{
-        char buf[128];
-        sprintf(buf, "M%d S%d\n", wait ? 109 : 104, temperature);
-        m_gcode += buf;
+        m_gcode += "M" + std::to_string(wait ? 109 : 104) + " S" + std::to_string(temperature) + "\n";
         return *this;
-	};
+    }
 
     // Wait for a period of time (seconds).
 	WipeTowerWriter& wait(float time)
 	{
-        if (time==0)
+        if (time==0.f)
             return *this;
 		char buf[128];
 		sprintf(buf, "G4 S%.3f\n", time);
 		m_gcode += buf;
 		return *this;
-	};
+    }
 
 	// Set speed factor override percentage.
 	WipeTowerWriter& speed_override(int speed)
 	{
-		char buf[128];
-		sprintf(buf, "M220 S%d\n", speed);
-		m_gcode += buf;
+        m_gcode += "M220 S" + std::to_string(speed) + "\n";
 		return *this;
-	};
+    }
 
 	// Let the firmware back up the active speed override value.
 	WipeTowerWriter& speed_override_backup()
-	{
-		m_gcode += "M220 B\n";
+    {
+        // This is only supported by Prusa at this point (https://github.com/prusa3d/PrusaSlicer/issues/3114)
+        if (m_gcode_flavor == gcfMarlin)
+            m_gcode += "M220 B\n";
 		return *this;
-	};
+    }
 
 	// Let the firmware restore the active speed override value.
 	WipeTowerWriter& speed_override_restore()
 	{
-		m_gcode += "M220 R\n";
+        if (m_gcode_flavor == gcfMarlin)
+            m_gcode += "M220 R\n";
 		return *this;
-	};
+    }
 
 	// Set digital trimpot motor
 	WipeTowerWriter& set_extruder_trimpot(int current)
 	{
-		char buf[128];
         if (m_gcode_flavor == gcfRepRap)
-            sprintf(buf, "M906 E%d\n", current);
+            m_gcode += "M906 E";
         else
-            sprintf(buf, "M907 E%d\n", current);
-		m_gcode += buf;
+            m_gcode += "M907 E";
+        m_gcode += std::to_string(current) + "\n";
 		return *this;
-	};
+    }
 
 	WipeTowerWriter& flush_planner_queue()
 	{ 
@@ -373,28 +371,20 @@ public:
 	}
 
 	WipeTowerWriter& comment_with_value(const char *comment, int value)
-	{
-		char strvalue[64];
-		sprintf(strvalue, "%d", value);
-		m_gcode += std::string(";") + comment + strvalue + "\n";
+    {
+        m_gcode += std::string(";") + comment + std::to_string(value) + "\n";
 		return *this;
-	};
+    }
 
 
-	WipeTowerWriter& set_fan(unsigned int speed)
+    WipeTowerWriter& set_fan(unsigned speed)
 	{
 		if (speed == m_last_fan_speed)
 			return *this;
-				
 		if (speed == 0)
 			m_gcode += "M107\n";
-		else
-		{
-			m_gcode += "M106 S";
-			char buf[128];
-			sprintf(buf,"%u\n",(unsigned int)(255.0 * speed / 100.0));
-			m_gcode += buf;
-		}
+        else
+            m_gcode += "M106 S" + std::to_string(unsigned(255.0 * speed / 100.0)) + "\n";
 		m_last_fan_speed = speed;
 		return *this;
 	}
@@ -406,7 +396,7 @@ private:
 	Vec2f         m_current_pos;
 	float    	  m_current_z;
 	float 	  	  m_current_feedrate;
-	unsigned int  m_current_tool;
+    size_t        m_current_tool;
 	float 		  m_layer_height;
 	float 	  	  m_extrusion_flow;
 	bool		  m_preview_suppressed;
@@ -417,7 +407,7 @@ private:
 	float		  m_y_shift = 0.f;
 	float 		  m_wipe_tower_width = 0.f;
 	float		  m_wipe_tower_depth = 0.f;
-	float		  m_last_fan_speed = 0.f;
+    unsigned      m_last_fan_speed = 0;
     int           current_temp = -1;
     const float   m_default_analyzer_line_width;
     float         m_used_filament_length = 0.f;
@@ -478,12 +468,13 @@ private:
 WipeTower::WipeTower(const PrintConfig& config, const std::vector<std::vector<float>>& wiping_matrix, size_t initial_tool) :
     m_semm(config.single_extruder_multi_material.value),
     m_wipe_tower_pos(config.wipe_tower_x, config.wipe_tower_y),
-    m_wipe_tower_width(config.wipe_tower_width),
-    m_wipe_tower_rotation_angle(config.wipe_tower_rotation_angle),
+    m_wipe_tower_width(float(config.wipe_tower_width)),
+    m_wipe_tower_rotation_angle(float(config.wipe_tower_rotation_angle)),
     m_y_shift(0.f),
     m_z_pos(0.f),
     m_is_first_layer(false),
-    m_bridging(config.wipe_tower_bridging),
+    m_bridging(float(config.wipe_tower_bridging)),
+    m_no_sparse_layers(config.wipe_tower_no_sparse_layers),
     m_gcode_flavor(config.gcode_flavor),
     m_current_tool(initial_tool),
     wipe_volumes(wiping_matrix)
@@ -491,16 +482,19 @@ WipeTower::WipeTower(const PrintConfig& config, const std::vector<std::vector<fl
     // If this is a single extruder MM printer, we will use all the SE-specific config values.
     // Otherwise, the defaults will be used to turn off the SE stuff.
     if (m_semm) {
-        m_cooling_tube_retraction = config.cooling_tube_retraction;
-        m_cooling_tube_length = config.cooling_tube_length;
-        m_parking_pos_retraction = config.parking_pos_retraction;
-        m_extra_loading_move = config.extra_loading_move;
-        m_set_extruder_trimpot = config.high_current_on_filament_swap;
+        m_cooling_tube_retraction = float(config.cooling_tube_retraction);
+        m_cooling_tube_length     = float(config.cooling_tube_length);
+        m_parking_pos_retraction  = float(config.parking_pos_retraction);
+        m_extra_loading_move      = float(config.extra_loading_move);
+        m_set_extruder_trimpot    = config.high_current_on_filament_swap;
     }
     // Calculate where the priming lines should be - very naive test not detecting parallelograms or custom shapes
     const std::vector<Vec2d>& bed_points = config.bed_shape.values;
     m_bed_shape = (bed_points.size() == 4 ? RectangularBed : CircularBed);
-    m_bed_width = BoundingBoxf(bed_points).size().x();
+    m_bed_width = float(BoundingBoxf(bed_points).size().x());
+    m_bed_bottom_left = m_bed_shape == RectangularBed
+                  ? Vec2f(bed_points.front().x(), bed_points.front().y())
+                  : Vec2f::Zero();
 }
 
 
@@ -517,21 +511,21 @@ void WipeTower::set_extruder(size_t idx, const PrintConfig& config)
     // If this is a single extruder MM printer, we will use all the SE-specific config values.
     // Otherwise, the defaults will be used to turn off the SE stuff.
     if (m_semm) {
-        m_filpar[idx].loading_speed           = config.filament_loading_speed.get_at(idx);
-        m_filpar[idx].loading_speed_start     = config.filament_loading_speed_start.get_at(idx);
-        m_filpar[idx].unloading_speed         = config.filament_unloading_speed.get_at(idx);
-        m_filpar[idx].unloading_speed_start   = config.filament_unloading_speed_start.get_at(idx);
-        m_filpar[idx].delay                   = config.filament_toolchange_delay.get_at(idx);
+        m_filpar[idx].loading_speed           = float(config.filament_loading_speed.get_at(idx));
+        m_filpar[idx].loading_speed_start     = float(config.filament_loading_speed_start.get_at(idx));
+        m_filpar[idx].unloading_speed         = float(config.filament_unloading_speed.get_at(idx));
+        m_filpar[idx].unloading_speed_start   = float(config.filament_unloading_speed_start.get_at(idx));
+        m_filpar[idx].delay                   = float(config.filament_toolchange_delay.get_at(idx));
         m_filpar[idx].cooling_moves           = config.filament_cooling_moves.get_at(idx);
-        m_filpar[idx].cooling_initial_speed   = config.filament_cooling_initial_speed.get_at(idx);
-        m_filpar[idx].cooling_final_speed     = config.filament_cooling_final_speed.get_at(idx);
+        m_filpar[idx].cooling_initial_speed   = float(config.filament_cooling_initial_speed.get_at(idx));
+        m_filpar[idx].cooling_final_speed     = float(config.filament_cooling_final_speed.get_at(idx));
     }
 
     m_filpar[idx].filament_area = float((M_PI/4.f) * pow(config.filament_diameter.get_at(idx), 2)); // all extruders are assumed to have the same filament diameter at this point
-    float nozzle_diameter = config.nozzle_diameter.get_at(idx);
+    float nozzle_diameter = float(config.nozzle_diameter.get_at(idx));
     m_filpar[idx].nozzle_diameter = nozzle_diameter; // to be used in future with (non-single) multiextruder MM
 
-    float max_vol_speed = config.filament_max_volumetric_speed.get_at(idx);
+    float max_vol_speed = float(config.filament_max_volumetric_speed.get_at(idx));
     if (max_vol_speed!= 0.f)
         m_filpar[idx].max_e_speed = (max_vol_speed / filament_area());
 
@@ -560,7 +554,7 @@ std::vector<WipeTower::ToolChangeResult> WipeTower::prime(
 	const std::vector<unsigned int> &tools,
 	// If true, the last priming are will be the same as the other priming areas, and the rest of the wipe will be performed inside the wipe tower.
 	// If false, the last priming are will be large enough to wipe the last extruder sufficiently.
-	bool 						last_wipe_inside_wipe_tower)
+    bool 						/*last_wipe_inside_wipe_tower*/)
 {
 	this->set_layer(first_layer_height, first_layer_height, tools.size(), true, false);
 	this->m_current_tool 		= tools.front();
@@ -575,12 +569,14 @@ std::vector<WipeTower::ToolChangeResult> WipeTower::prime(
     // In case of a circular bed, place it so it goes across the diameter and hope it will fit
     if (m_bed_shape == CircularBed)
         cleaning_box.translate(-m_bed_width/2 + m_bed_width * 0.03f, -m_bed_width * 0.12f);
+    if (m_bed_shape == RectangularBed)
+        cleaning_box.translate(m_bed_bottom_left);
 
     std::vector<ToolChangeResult> results;
 
     // Iterate over all priming toolchanges and push respective ToolChangeResults into results vector.
     for (size_t idx_tool = 0; idx_tool < tools.size(); ++ idx_tool) {
-        int old_tool = m_current_tool;
+        size_t old_tool = m_current_tool;
 
         WipeTowerWriter writer(m_layer_height, m_perimeter_width, m_gcode_flavor, m_filpar);
         writer.set_extrusion_flow(m_extrusion_flow)
@@ -629,8 +625,8 @@ std::vector<WipeTower::ToolChangeResult> WipeTower::prime(
 
         ToolChangeResult result;
         result.priming      = true;
-        result.initial_tool = old_tool;
-        result.new_tool     = m_current_tool;
+        result.initial_tool = int(old_tool);
+        result.new_tool     = int(m_current_tool);
         result.print_z 	  	= this->m_z_pos;
         result.layer_height = this->m_layer_height;
         result.gcode   	  	= writer.gcode();
@@ -665,12 +661,12 @@ std::vector<WipeTower::ToolChangeResult> WipeTower::prime(
 	return results;
 }
 
-WipeTower::ToolChangeResult WipeTower::tool_change(unsigned int tool, bool last_in_layer)
+WipeTower::ToolChangeResult WipeTower::tool_change(size_t tool, bool last_in_layer)
 {
 	if ( m_print_brim )
 		return toolchange_Brim();
 
-    int old_tool = m_current_tool;
+    size_t old_tool = m_current_tool;
 
 	float wipe_area = 0.f;
 	bool last_change_in_layer = false;
@@ -695,7 +691,7 @@ WipeTower::ToolChangeResult WipeTower::tool_change(unsigned int tool, bool last_
 	box_coordinates cleaning_box(
 		Vec2f(m_perimeter_width / 2.f, m_perimeter_width / 2.f),
 		m_wipe_tower_width - m_perimeter_width,
-		(tool != (unsigned int)(-1) ? /*m_layer_info->depth*/wipe_area+m_depth_traversed-0.5*m_perimeter_width
+        (tool != (unsigned int)(-1) ? /*m_layer_info->depth*/wipe_area+m_depth_traversed-0.5f*m_perimeter_width
                                     : m_wipe_tower_depth-m_perimeter_width));
 
 	WipeTowerWriter writer(m_layer_height, m_perimeter_width, m_gcode_flavor, m_filpar);
@@ -711,7 +707,7 @@ WipeTower::ToolChangeResult WipeTower::tool_change(unsigned int tool, bool last_
         writer.append(std::string("; material : " + (m_current_tool < m_filpar.size() ? m_filpar[m_current_tool].material : "(NONE)") + " -> " + m_filpar[tool].material + "\n").c_str())
               .append(";--------------------\n");
 
-	writer.speed_override_backup();
+    writer.speed_override_backup();
 	writer.speed_override(100);
 
 	Vec2f initial_position = cleaning_box.ld + Vec2f(0.f, m_depth_traversed);
@@ -761,10 +757,10 @@ WipeTower::ToolChangeResult WipeTower::tool_change(unsigned int tool, bool last_
     if (m_current_tool < m_used_filament_length.size())
         m_used_filament_length[m_current_tool] += writer.get_and_reset_used_filament_length();
 
-	ToolChangeResult result;
+    ToolChangeResult result;
     result.priming      = false;
-    result.initial_tool = old_tool;
-    result.new_tool     = m_current_tool;
+    result.initial_tool = int(old_tool);
+    result.new_tool     = int(m_current_tool);
 	result.print_z 	  	= this->m_z_pos;
 	result.layer_height = this->m_layer_height;
 	result.gcode   	  	= writer.gcode();
@@ -777,7 +773,7 @@ WipeTower::ToolChangeResult WipeTower::tool_change(unsigned int tool, bool last_
 
 WipeTower::ToolChangeResult WipeTower::toolchange_Brim(bool sideOnly, float y_offset)
 {
-    int old_tool = m_current_tool;
+    size_t old_tool = m_current_tool;
 
 	const box_coordinates wipeTower_box(
 		Vec2f::Zero(),
@@ -800,8 +796,10 @@ WipeTower::ToolChangeResult WipeTower::toolchange_Brim(bool sideOnly, float y_of
     // The tool is supposed to be active and primed at the time when the wipe tower brim is extruded.
     // Extrude 4 rounds of a brim around the future wipe tower.
     box_coordinates box(wipeTower_box);
+    // the brim shall have 'normal' spacing with no extra void space
+    float spacing = m_perimeter_width - m_layer_height*float(1.-M_PI_4);
     for (size_t i = 0; i < 4; ++ i) {
-        box.expand(m_perimeter_width - m_layer_height*(1.f-M_PI_4)); // the brim shall have 'normal' spacing with no extra void space
+        box.expand(spacing);
         writer.travel (box.ld, 7000)
                 .extrude(box.lu, 2100).extrude(box.ru)
                 .extrude(box.rd      ).extrude(box.ld);
@@ -813,16 +811,20 @@ WipeTower::ToolChangeResult WipeTower::toolchange_Brim(bool sideOnly, float y_of
     writer.append("; CP WIPE TOWER FIRST LAYER BRIM END\n"
                   ";-----------------------------------\n");
 
+    // Save actual brim width to be later passed to the Print object, which will use it
+    // for skirt calculation and pass it to GLCanvas for precise preview box
+    m_wipe_tower_brim_width = wipeTower_box.ld.x() - box.ld.x() + spacing/2.f;
+
     m_print_brim = false;  // Mark the brim as extruded
 
     // Ask our writer about how much material was consumed:
     if (m_current_tool < m_used_filament_length.size())
     	m_used_filament_length[m_current_tool] += writer.get_and_reset_used_filament_length();
 
-	ToolChangeResult result;
+    ToolChangeResult result;
     result.priming      = false;
-    result.initial_tool = old_tool;
-    result.new_tool     = m_current_tool;
+    result.initial_tool = int(old_tool);
+    result.new_tool     = int(m_current_tool);
 	result.print_z 	  	= this->m_z_pos;
 	result.layer_height = this->m_layer_height;
 	result.gcode   	  	= writer.gcode();
@@ -910,8 +912,8 @@ void WipeTower::toolchange_Unload(
         const float x = volume_to_length(m_filpar[m_current_tool].ramming_speed[i] * 0.25f, line_width, m_layer_height);
         const float e = m_filpar[m_current_tool].ramming_speed[i] * 0.25f / filament_area(); // transform volume per sec to E move;
         const float dist = std::min(x - e_done, remaining);		  // distance to travel for either the next 0.25s, or to the next turnaround
-        const float actual_time = dist/x * 0.25;
-        writer.ram(writer.x(), writer.x() + (m_left_to_right ? 1.f : -1.f) * dist, 0, 0, e * (dist / x), dist / (actual_time / 60.));
+        const float actual_time = dist/x * 0.25f;
+        writer.ram(writer.x(), writer.x() + (m_left_to_right ? 1.f : -1.f) * dist, 0.f, 0.f, e * (dist / x), dist / (actual_time / 60.f));
         remaining -= dist;
 
 		if (remaining < WT_EPSILON)	{ // we reached a turning point
@@ -994,8 +996,8 @@ void WipeTower::toolchange_Unload(
 // Change the tool, set a speed override for soluble and flex materials.
 void WipeTower::toolchange_Change(
 	WipeTowerWriter &writer,
-	const unsigned int 	new_tool, 
-	const std::string&  new_material)
+    const size_t 	new_tool,
+    const std::string&  new_material)
 {
     // Ask the writer about how much of the old filament we consumed:
     if (m_current_tool < m_used_filament_length.size())
@@ -1007,9 +1009,10 @@ void WipeTower::toolchange_Change(
     writer.append("[toolchange_gcode]\n");
 
     // Travel to where we assume we are. Custom toolchange or some special T code handling (parking extruder etc)
-    // gcode could have left the extruder somewhere, we cannot just start extruding.
-	Vec2f current_pos = writer.pos_rotated();
-    writer.append(std::string("G1 X") + std::to_string(current_pos.x()) +  " Y" + std::to_string(current_pos.y()) +  "\n");
+    // gcode could have left the extruder somewhere, we cannot just start extruding. We should also inform the
+    // postprocessor that we absolutely want to have this in the gcode, even if it thought it is the same as before.
+    Vec2f current_pos = writer.pos_rotated();
+    writer.append(std::string("G1 X") + std::to_string(current_pos.x()) +  " Y" + std::to_string(current_pos.y()) + never_skip_tag() + "\n");
 
     // The toolchange Tn command will be inserted later, only in case that the user does
     // not provide a custom toolchange gcode.
@@ -1091,21 +1094,21 @@ void WipeTower::toolchange_Wipe(
 
 		float traversed_x = writer.x();
 		if (m_left_to_right)
-			writer.extrude(xr - (i % 4 == 0 ? 0 : 1.5*m_perimeter_width), writer.y(), wipe_speed * wipe_coeff);
+            writer.extrude(xr - (i % 4 == 0 ? 0 : 1.5f*m_perimeter_width), writer.y(), wipe_speed * wipe_coeff);
 		else
-			writer.extrude(xl + (i % 4 == 1 ? 0 : 1.5*m_perimeter_width), writer.y(), wipe_speed * wipe_coeff);
+            writer.extrude(xl + (i % 4 == 1 ? 0 : 1.5f*m_perimeter_width), writer.y(), wipe_speed * wipe_coeff);
 
-        if (writer.y()+EPSILON > cleaning_box.lu.y()-0.5f*m_perimeter_width)
+        if (writer.y()+float(EPSILON) > cleaning_box.lu.y()-0.5f*m_perimeter_width)
             break;		// in case next line would not fit
 
 		traversed_x -= writer.x();
-		x_to_wipe -= fabs(traversed_x);
+        x_to_wipe -= std::abs(traversed_x);
 		if (x_to_wipe < WT_EPSILON) {
-			writer.travel(m_left_to_right ? xl + 1.5*m_perimeter_width : xr - 1.5*m_perimeter_width, writer.y(), 7200);
+            writer.travel(m_left_to_right ? xl + 1.5f*m_perimeter_width : xr - 1.5f*m_perimeter_width, writer.y(), 7200);
 			break;
 		}
 		// stepping to the next line:
-		writer.extrude(writer.x() + (i % 4 == 0 ? -1.f : (i % 4 == 1 ? 1.f : 0.f)) * 1.5*m_perimeter_width, writer.y() + dy);
+        writer.extrude(writer.x() + (i % 4 == 0 ? -1.f : (i % 4 == 1 ? 1.f : 0.f)) * 1.5f*m_perimeter_width, writer.y() + dy);
 		m_left_to_right = !m_left_to_right;
 	}
 
@@ -1128,7 +1131,7 @@ WipeTower::ToolChangeResult WipeTower::finish_layer()
 	// Otherwise the caller would likely travel to the wipe tower in vain.
 	assert(! this->layer_finished());
 
-    int old_tool = m_current_tool;
+    size_t old_tool = m_current_tool;
 
 	WipeTowerWriter writer(m_layer_height, m_perimeter_width, m_gcode_flavor, m_filpar);
 	writer.set_extrusion_flow(m_extrusion_flow)
@@ -1149,9 +1152,10 @@ WipeTower::ToolChangeResult WipeTower::finish_layer()
     writer.set_initial_position((m_left_to_right ? fill_box.ru : fill_box.lu), // so there is never a diagonal travel
                                  m_wipe_tower_width, m_wipe_tower_depth, m_internal_rotation);
 
+    bool toolchanges_on_layer = m_layer_info->toolchanges_depth() > WT_EPSILON;
 	box_coordinates box = fill_box;
     for (int i=0;i<2;++i) {
-        if (m_layer_info->toolchanges_depth() < WT_EPSILON) { // there were no toolchanges on this layer
+        if (! toolchanges_on_layer) {
             if (i==0) box.expand(m_perimeter_width);
             else box.expand(-m_perimeter_width);
         }
@@ -1176,7 +1180,7 @@ WipeTower::ToolChangeResult WipeTower::finish_layer()
                 writer.extrude(box.rd.x() - m_perimeter_width / 2.f, writer.y() + 0.5f * step);
                 writer.extrude(box.ld.x() + m_perimeter_width / 2.f, writer.y());
             }
-            writer.travel(box.rd.x()-m_perimeter_width/2.f,writer.y()); // wipe the nozzle
+        writer.travel(box.rd.x()-m_perimeter_width/2.f,writer.y()); // wipe the nozzle
     }
     else {  // Extrude a sparse infill to support the material to be printed above.
         const float dy = (fill_box.lu.y() - fill_box.ld.y() - m_perimeter_width);
@@ -1188,7 +1192,7 @@ WipeTower::ToolChangeResult WipeTower::finish_layer()
             writer.travel(fill_box.ld + Vec2f(m_perimeter_width * 2, 0.f))
                   .extrude(fill_box.lu + Vec2f(m_perimeter_width * 2, 0.f), 2900 * speed_factor);
 
-            const int n = 1+(right-left)/(m_bridging);
+            const int n = 1+int((right-left)/m_bridging);
             const float dx = (right-left)/n;
             for (int i=1;i<=n;++i) {
                 float x=left+dx*i;
@@ -1205,14 +1209,17 @@ WipeTower::ToolChangeResult WipeTower::finish_layer()
 
     m_depth_traversed = m_wipe_tower_depth-m_perimeter_width;
 
-    // Ask our writer about how much material was consumed:
-    if (m_current_tool < m_used_filament_length.size())
-        m_used_filament_length[m_current_tool] += writer.get_and_reset_used_filament_length();
 
-	ToolChangeResult result;
+    // Ask our writer about how much material was consumed.
+    // Skip this in case the layer is sparse and config option to not print sparse layers is enabled.
+    if (! m_no_sparse_layers || toolchanges_on_layer)
+        if (m_current_tool < m_used_filament_length.size())
+            m_used_filament_length[m_current_tool] += writer.get_and_reset_used_filament_length();
+
+    ToolChangeResult result;
     result.priming      = false;
-    result.initial_tool = old_tool;
-    result.new_tool     = m_current_tool;
+    result.initial_tool = int(old_tool);
+    result.new_tool     = int(m_current_tool);
 	result.print_z 	  	= this->m_z_pos;
 	result.layer_height = this->m_layer_height;
 	result.gcode   	  	= writer.gcode();
@@ -1267,7 +1274,7 @@ void WipeTower::plan_tower()
 	for (auto& layer : m_plan)
 		layer.depth = 0.f;
 	
-	for (int layer_index = m_plan.size() - 1; layer_index >= 0; --layer_index)
+    for (int layer_index = int(m_plan.size()) - 1; layer_index >= 0; --layer_index)
 	{
 		float this_layer_depth = std::max(m_plan[layer_index].depth, m_plan[layer_index].toolchanges_depth());
 		m_plan[layer_index].depth = this_layer_depth;
