@@ -4,6 +4,7 @@
 #include "libslic3r.h"
 #include "PrintConfig.hpp"
 #include "GCodeReader.hpp"
+#include "CustomGCode.hpp"
 
 #define ENABLE_MOVE_STATS 0
 
@@ -23,6 +24,7 @@ namespace Slic3r {
         static const std::string Silent_Last_M73_Output_Placeholder_Tag;
 
         static const std::string Color_Change_Tag;
+        static const std::string Pause_Print_Tag;
 
         enum EMode : unsigned char
         {
@@ -55,6 +57,7 @@ namespace Slic3r {
         struct Axis
         {
             float position;         // mm
+            float origin;           // mm
             float max_feedrate;     // mm/s
             float max_acceleration; // mm/s^2
             float max_jerk;         // mm/s
@@ -213,9 +216,19 @@ namespace Slic3r {
         typedef std::map<Block::EMoveType, MoveStats> MovesStatsMap;
 #endif // ENABLE_MOVE_STATS
 
+    public:
         typedef std::pair<unsigned int, unsigned int> G1LineIdToBlockId;
         typedef std::vector<G1LineIdToBlockId> G1LineIdToBlockIdMap;
 
+        struct PostProcessData
+        {
+            const G1LineIdToBlockIdMap& g1_line_ids;
+            const BlocksList& blocks;
+            float time;
+
+            PostProcessData(const G1LineIdToBlockIdMap& g1_line_ids, const BlocksList& blocks, float time) : g1_line_ids(g1_line_ids), blocks(blocks), time(time) {}
+        };
+
     private:
         EMode m_mode;
         GCodeReader m_parser;
@@ -229,10 +242,10 @@ namespace Slic3r {
         int m_last_st_synchronized_block_id;
         float m_time; // s
 
-        // data to calculate color print times
-        bool m_needs_color_times;
-        std::vector<float> m_color_times;
-        float m_color_time_cache;
+        // data to calculate custom code times
+        bool m_needs_custom_gcode_times;
+        std::vector<std::pair<CustomGcodeType, float>> m_custom_gcode_times;
+        float m_custom_gcode_time_cache;
 
 #if ENABLE_MOVE_STATS
         MovesStatsMap _moves_stats;
@@ -263,14 +276,17 @@ namespace Slic3r {
         void calculate_time_from_lines(const std::vector<std::string>& gcode_lines);
 
         // Process the gcode contained in the file with the given filename, 
-        // placing in it new lines (M73) containing the remaining time, at the given interval in seconds
-        // and saving the result back in the same file
-        // This time estimator should have been already used to calculate the time estimate for the gcode
-        // contained in the given file before to call this method
-        bool post_process_remaining_times(const std::string& filename, float interval_sec);
+        // replacing placeholders with correspondent new lines M73
+        // placing new lines M73 (containing the remaining time) where needed (in dependence of the given interval in seconds)
+        // and removing working tags (as those used for color changes)
+        // if normal_mode == nullptr no M73 line will be added for normal mode
+        // if silent_mode == nullptr no M73 line will be added for silent mode
+        static bool post_process(const std::string& filename, float interval_sec, const PostProcessData* const normal_mode, const PostProcessData* const silent_mode);
 
         // Set current position on the given axis with the given value
         void set_axis_position(EAxis axis, float position);
+        // Set current origin on the given axis with the given value
+        void set_axis_origin(EAxis axis, float position);
 
         void set_axis_max_feedrate(EAxis axis, float feedrate_mm_sec);
         void set_axis_max_acceleration(EAxis axis, float acceleration);
@@ -278,6 +294,8 @@ namespace Slic3r {
 
         // Returns current position on the given axis
         float get_axis_position(EAxis axis) const;
+        // Returns current origin on the given axis
+        float get_axis_origin(EAxis axis) const;
 
         float get_axis_max_feedrate(EAxis axis) const;
         float get_axis_max_acceleration(EAxis axis) const;
@@ -326,6 +344,8 @@ namespace Slic3r {
         void increment_g1_line_id();
         void reset_g1_line_id();
 
+        void set_extrusion_axis(char axis) { m_parser.set_extrusion_axis(axis); }
+
         void set_extruder_id(unsigned int id);
         unsigned int get_extruder_id() const;
         void reset_extruder_id();
@@ -345,11 +365,14 @@ namespace Slic3r {
         // Returns the estimated time, in format DDd HHh MMm SSs
         std::string get_time_dhms() const;
 
+        // Returns the estimated time, in format DDd HHh MMm
+        std::string get_time_dhm() const;
+
         // Returns the estimated time, in minutes (integer)
         std::string get_time_minutes() const;
 
-        // Returns the estimated time, in seconds, for each color
-        std::vector<float> get_color_times() const;
+        // Returns the estimated time, in seconds, for each custom gcode
+        std::vector<std::pair<CustomGcodeType, float>> get_custom_gcode_times() const;
 
         // Returns the estimated time, in format DDd HHh MMm SSs, for each color
         // If include_remaining==true the strings will be formatted as: "time for color (remaining time at color start)"
@@ -359,9 +382,15 @@ namespace Slic3r {
         // If include_remaining==true the strings will be formatted as: "time for color (remaining time at color start)"
         std::vector<std::string> get_color_times_minutes(bool include_remaining) const;
 
+        // Returns the estimated time, in format DDd HHh MMm, for each custom_gcode
+        // If include_remaining==true the strings will be formatted as: "time for custom_gcode (remaining time at color start)"
+        std::vector<std::pair<CustomGcodeType, std::string>> get_custom_gcode_times_dhm(bool include_remaining) const;
+
         // Return an estimate of the memory consumed by the time estimator.
         size_t memory_used() const;
 
+        PostProcessData get_post_process_data() const { return PostProcessData(m_g1_line_ids, m_blocks, m_time); }
+
     private:
         void _reset();
         void _reset_time();
@@ -437,8 +466,8 @@ namespace Slic3r {
         // Returns true if any tag has been processed
         bool _process_tags(const GCodeReader::GCodeLine& line);
 
-        // Processes color change tag
-        void _process_color_change_tag();
+        // Processes ColorChangeTag and PausePrintTag
+        void _process_custom_gcode_tag(CustomGcodeType code);
 
         // Simulates firmware st_synchronize() call
         void _simulate_st_synchronize();
@@ -453,6 +482,8 @@ namespace Slic3r {
 
         // Returns the given time is seconds in format DDd HHh MMm SSs
         static std::string _get_time_dhms(float time_in_secs);
+        // Returns the given time is minutes in format DDd HHh MMm
+        static std::string _get_time_dhm(float time_in_secs);
 
         // Returns the given, in minutes (integer)
         static std::string _get_time_minutes(float time_in_secs);
