@@ -11,8 +11,11 @@
 #include <cereal/access.hpp>
 
 #include "boost/polygon/voronoi.hpp"
-using boost::polygon::voronoi_builder;
-using boost::polygon::voronoi_diagram;
+
+namespace ClipperLib {
+class PolyNode;
+using PolyNodes = std::vector<PolyNode*>;
+}
 
 namespace Slic3r { namespace Geometry {
 
@@ -134,13 +137,83 @@ inline bool segments_intersect(
 		   segments_could_intersect(jp1, jp2, ip1, ip2) <= 0;
 }
 
+// Based on Liang-Barsky function by Daniel White @ http://www.skytopia.com/project/articles/compsci/clipping.html
+template<typename T>
+bool liang_barsky_line_clipping(
+	// Start and end points of the source line, result will be stored there as well.
+	Eigen::Matrix<T, 2, 1, Eigen::DontAlign> 						&x0,
+	Eigen::Matrix<T, 2, 1, Eigen::DontAlign> 						&x1,
+	// Bounding box to clip with.
+	const BoundingBoxBase<Eigen::Matrix<T, 2, 1, Eigen::DontAlign>> &bbox)
+{
+    Eigen::Matrix<T, 2, 1, Eigen::DontAlign> v = x1 - x0;
+    double t0 = 0.0;
+    double t1 = 1.0;
+
+	// Traverse through left, right, bottom, top edges.
+    for (int edge = 0; edge < 4; ++ edge)
+    {
+		double p, q;
+    	switch (edge) {
+    	case 0:	 p = - v.x();    q = - bbox.min.x() + x0.x();	break;
+        case 1:  p =   v.x();    q =   bbox.max.x() - x0.x();	break;
+        case 2:  p = - v.y();    q = - bbox.min.y() + x0.y();	break;
+        default: p =   v.y();    q =   bbox.max.y() - x0.y();   break;
+    	}
+        
+		if (p == 0) {
+			if (q < 0)
+				// Line parallel to the bounding box edge is fully outside of the bounding box.
+				return false;
+			// else don't clip
+		} else {
+	        double r = q / p;
+			if (p < 0) {
+				if (r > t1)
+            		// Fully clipped.
+            		return false;
+				if (r > t0)
+            		// Partially clipped.
+            		t0 = r;
+			} else {
+				assert(p > 0);
+				if (r < t0)
+            		// Fully clipped.
+            		return false;
+				if (r < t1)
+            		// Partially clipped.
+            		t1 = r;
+			}
+        }
+    }
+
+    // Clipped successfully.
+    x1  = x0 + t1 * v;
+    x0 += t0 * v;
+    return true;
+}
+
+// Based on Liang-Barsky function by Daniel White @ http://www.skytopia.com/project/articles/compsci/clipping.html
+template<typename T>
+bool liang_barsky_line_clipping(
+	// Start and end points of the source line.
+	const Eigen::Matrix<T, 2, 1, Eigen::DontAlign> 					&x0src,
+	const Eigen::Matrix<T, 2, 1, Eigen::DontAlign> 					&x1src,
+	// Bounding box to clip with.
+	const BoundingBoxBase<Eigen::Matrix<T, 2, 1, Eigen::DontAlign>> &bbox,
+	// Start and end points of the clipped line.
+	Eigen::Matrix<T, 2, 1, Eigen::DontAlign> 						&x0clip,
+	Eigen::Matrix<T, 2, 1, Eigen::DontAlign> 						&x1clip)
+{
+	x0clip = x0src;
+	x1clip = x1src;
+	return liang_barsky_line_clipping(x0clip, x1clip, bbox);
+}
+
 Pointf3s convex_hull(Pointf3s points);
 Polygon convex_hull(Points points);
 Polygon convex_hull(const Polygons &polygons);
 
-void chained_path(const Points &points, std::vector<Points::size_type> &retval, Point start_near);
-void chained_path(const Points &points, std::vector<Points::size_type> &retval);
-template<class T> void chained_path_items(Points &points, T &items, T &retval);
 bool directions_parallel(double angle1, double angle2, double max_diff = 0);
 template<class T> bool contains(const std::vector<T> &vector, const Point &point);
 template<typename T> T rad2deg(T angle) { return T(180.0) * angle / T(PI); }
@@ -160,6 +233,15 @@ template<typename T> T angle_to_0_2PI(T angle)
 
     return angle;
 }
+
+/// Find the center of the circle corresponding to the vector of Points as an arc.
+Point circle_taubin_newton(const Points::const_iterator& input_start, const Points::const_iterator& input_end, size_t cycles = 20);
+inline Point circle_taubin_newton(const Points& input, size_t cycles = 20) { return circle_taubin_newton(input.cbegin(), input.cend(), cycles); }
+
+/// Find the center of the circle corresponding to the vector of Pointfs as an arc.
+Vec2d circle_taubin_newton(const Vec2ds::const_iterator& input_start, const Vec2ds::const_iterator& input_end, size_t cycles = 20);
+inline Vec2d circle_taubin_newton(const Vec2ds& input, size_t cycles = 20) { return circle_taubin_newton(input.cbegin(), input.cend(), cycles); }
+
 void simplify_polygons(const Polygons &polygons, double tolerance, Polygons* retval);
 
 double linint(double value, double oldmin, double oldmax, double newmin, double newmax);
@@ -181,7 +263,7 @@ class MedialAxis {
     void build(Polylines* polylines);
     
     private:
-    class VD : public voronoi_diagram<double> {
+    class VD : public boost::polygon::voronoi_diagram<double> {
     public:
         typedef double                                          coord_type;
         typedef boost::polygon::point_data<coordinate_type>     point_type;
@@ -302,6 +384,9 @@ private:
 	}
 };
 
+// For parsing a transformation matrix from 3MF / AMF.
+extern Transform3d transform3d_from_string(const std::string& transform_str);
+
 // Rotation when going from the first coordinate system with rotation rot_xyz_from applied
 // to a coordinate system with rot_xyz_to applied.
 extern Eigen::Quaterniond rotation_xyz_diff(const Vec3d &rot_xyz_from, const Vec3d &rot_xyz_to);
