@@ -86,10 +86,7 @@ bool Line::intersection(const Line &l2, Point *intersection) const
     const Line  &l1  = *this;
     const Vec2d  v1  = (l1.b - l1.a).cast<double>();
     const Vec2d  v2  = (l2.b - l2.a).cast<double>();
-    const Vec2d  v12 = (l1.a - l2.a).cast<double>();
     double       denom  = cross2(v1, v2);
-    double       nume_a = cross2(v2, v12);
-    double       nume_b = cross2(v1, v12);
     if (fabs(denom) < EPSILON)
 #if 0
         // Lines are collinear. Return true if they are coincident (overlappign).
@@ -97,6 +94,9 @@ bool Line::intersection(const Line &l2, Point *intersection) const
 #else
         return false;
 #endif
+    const Vec2d v12 = (l1.a - l2.a).cast<double>();
+    double nume_a = cross2(v2, v12);
+    double nume_b = cross2(v1, v12);
     double t1 = nume_a / denom;
     double t2 = nume_b / denom;
     if (t1 >= 0 && t1 <= 1.0f && t2 >= 0 && t2 <= 1.0f) {
@@ -107,6 +107,17 @@ bool Line::intersection(const Line &l2, Point *intersection) const
     return false;  // not intersecting
 }
 
+bool Line::clip_with_bbox(const BoundingBox &bbox)
+{
+	Vec2d x0clip, x1clip;
+	bool result = Geometry::liang_barsky_line_clipping<double>(this->a.cast<double>(), this->b.cast<double>(), BoundingBoxf(bbox.min.cast<double>(), bbox.max.cast<double>()), x0clip, x1clip);
+	if (result) {
+		this->a = x0clip.cast<coord_t>();
+		this->b = x1clip.cast<coord_t>();
+	}
+	return result;
+}
+
 Vec3d Linef3::intersect_plane(double z) const
 {
     auto   v = (this->b - this->a).cast<double>();
