@@ -62,9 +62,10 @@ public:
 
     operator Polylines() const;
     operator Line() const;
-    Point last_point() const;
-    Point leftmost_point() const;
-    virtual Lines lines() const;
+    const Point& last_point() const override { return this->points.back(); }
+
+    const Point& leftmost_point() const;
+    Lines lines() const override;
     void clip_end(double distance);
     void clip_start(double distance);
     void extend_end(double distance);
@@ -76,6 +77,15 @@ public:
     bool is_straight() const;
 };
 
+// Don't use this class in production code, it is used exclusively by the Perl binding for unit tests!
+#ifdef PERL_UCHAR_MIN
+class PolylineCollection
+{
+public:
+    Polylines polylines;
+};
+#endif /* PERL_UCHAR_MIN */
+
 extern BoundingBox get_extents(const Polyline &polyline);
 extern BoundingBox get_extents(const Polylines &polylines);
 
@@ -128,6 +138,8 @@ inline void polylines_append(Polylines &dst, Polylines &&src)
     }
 }
 
+const Point& leftmost_point(const Polylines &polylines);
+
 bool remove_degenerate(Polylines &polylines);
 
 class ThickPolyline : public Polyline {
