@@ -15,6 +15,8 @@
 #include "GCode/ThumbnailData.hpp"
 #endif // ENABLE_THUMBNAIL_GENERATOR
 
+#include "libslic3r.h"
+
 namespace Slic3r {
 
 class Print;
@@ -22,11 +24,23 @@ class PrintObject;
 class ModelObject;
 class GCode;
 class GCodePreviewData;
+enum class SlicingMode : uint32_t;
 
 // Print step IDs for keeping track of the print state.
 enum PrintStep {
-    psSkirt, psBrim, psWipeTower, psGCodeExport, psCount,
+    psSkirt, 
+    psBrim,
+    // Synonym for the last step before the Wipe Tower / Tool Ordering, for the G-code preview slider to understand that 
+    // all the extrusions are there for the layer slider to add color changes etc.
+    psExtrusionPaths = psBrim,
+    psWipeTower,
+    // psToolOrdering is a synonym to psWipeTower, as the Wipe Tower calculates and modifies the ToolOrdering,
+    // while if printing without the Wipe Tower, the ToolOrdering is calculated as well.
+    psToolOrdering = psWipeTower,
+    psGCodeExport,
+    psCount,
 };
+
 enum PrintObjectStep {
     posSlice, posPerimeters, posPrepareInfill,
     posInfill, posSupportMaterial, posCount,
@@ -42,13 +56,15 @@ class PrintRegion
 public:
     const Print*                print() const { return m_print; }
     const PrintRegionConfig&    config() const { return m_config; }
+	// 1-based extruder identifier for this region and role.
+	unsigned int 				extruder(FlowRole role) const;
     Flow                        flow(FlowRole role, double layer_height, bool bridge, bool first_layer, double width, const PrintObject &object) const;
     // Average diameter of nozzles participating on extruding this region.
     coordf_t                    nozzle_dmr_avg(const PrintConfig &print_config) const;
     // Average diameter of nozzles participating on extruding this region.
     coordf_t                    bridging_height_avg(const PrintConfig &print_config) const;
 
-    // Collect extruder indices used to print this region's object.
+    // Collect 0-based extruder indices used to print this region's object.
 	void                        collect_object_printing_extruders(std::vector<unsigned int> &object_extruders) const;
 	static void                 collect_object_printing_extruders(const PrintConfig &print_config, const PrintRegionConfig &region_config, std::vector<unsigned int> &object_extruders);
 
@@ -69,7 +85,7 @@ private:
     
     PrintRegion(Print* print) : m_refcnt(0), m_print(print) {}
     PrintRegion(Print* print, const PrintRegionConfig &config) : m_refcnt(0), m_print(print), m_config(config) {}
-    ~PrintRegion() {}
+    ~PrintRegion() = default;
 };
 
 
@@ -77,6 +93,21 @@ typedef std::vector<Layer*> LayerPtrs;
 typedef std::vector<SupportLayer*> SupportLayerPtrs;
 class BoundingBoxf3;        // TODO: for temporary constructor parameter
 
+// Single instance of a PrintObject.
+// As multiple PrintObjects may be generated for a single ModelObject (their instances differ in rotation around Z),
+// ModelObject's instancess will be distributed among these multiple PrintObjects.
+struct PrintInstance
+{
+    // Parent PrintObject
+    PrintObject 		*print_object;
+    // Source ModelInstance of a ModelObject, for which this print_object was created.
+	const ModelInstance *model_instance;
+	// Shift of this instance's center into the world coordinates.
+	Point 				 shift;
+};
+
+typedef std::vector<PrintInstance> PrintInstances;
+
 class PrintObject : public PrintObjectBaseWithState<Print, PrintObjectStep, posCount>
 {
 private: // Prevents erroneous use by other classes.
@@ -86,20 +117,22 @@ public:
     // vector of (layer height ranges and vectors of volume ids), indexed by region_id
     std::vector<std::vector<std::pair<t_layer_height_range, int>>> region_volumes;
 
-    // this is set to true when LayerRegion->slices is split in top/internal/bottom
-    // so that next call to make_perimeters() performs a union() before computing loops
-    bool                    typed_slices;
-
-    Vec3crd                 size;           // XYZ in scaled coordinates
-
+    // Size of an object: XYZ in scaled coordinates. The size might not be quite snug in XY plane.
+    const Vec3crd&          size() const			{ return m_size; }
     const PrintObjectConfig& config() const         { return m_config; }    
     const LayerPtrs&        layers() const          { return m_layers; }
     const SupportLayerPtrs& support_layers() const  { return m_support_layers; }
     const Transform3d&      trafo() const           { return m_trafo; }
-    const Points&           copies() const          { return m_copies; }
+    const PrintInstances&   instances() const       { return m_instances; }
 
-    // since the object is aligned to origin, bounding box coincides with size
-    BoundingBox bounding_box() const { return BoundingBox(Point(0,0), to_2d(this->size)); }
+    // Bounding box is used to align the object infill patterns, and to calculate attractor for the rear seam.
+    // The bounding box may not be quite snug.
+    BoundingBox             bounding_box()    const { return BoundingBox(Point(- m_size.x() / 2, - m_size.y() / 2), Point(m_size.x() / 2, m_size.y() / 2)); }
+    // Height is used for slicing, for sorting the objects by height for sequential printing and for checking vertical clearence in sequential print mode.
+    // The height is snug.
+    coord_t 				height() 		  const { return m_size.z(); }
+    // Centering offset of the sliced mesh from the scaled and rotated mesh of the model.
+    const Point& 			center_offset()   const { return m_center_offset; }
 
     // adds region_id, too, if necessary
     void add_region_volume(unsigned int region_id, int volume_id, const t_layer_height_range &layer_range) {
@@ -110,11 +143,24 @@ public:
     // This is the *total* layer count (including support layers)
     // this value is not supposed to be compared with Layer::id
     // since they have different semantics.
-    size_t total_layer_count() const { return this->layer_count() + this->support_layer_count(); }
-    size_t layer_count() const { return m_layers.size(); }
-    void clear_layers();
-    Layer* get_layer(int idx) { return m_layers[idx]; }
-    const Layer* get_layer(int idx) const { return m_layers[idx]; }
+    size_t 			total_layer_count() const { return this->layer_count() + this->support_layer_count(); }
+    size_t 			layer_count() const { return m_layers.size(); }
+    void 			clear_layers();
+    const Layer* 	get_layer(int idx) const { return m_layers[idx]; }
+    Layer* 			get_layer(int idx) 		 { return m_layers[idx]; }
+    // Get a layer exactly at print_z.
+    const Layer*	get_layer_at_printz(coordf_t print_z) const {
+        auto it = Slic3r::lower_bound_by_predicate(m_layers.begin(), m_layers.end(), [print_z](const Layer *layer) { return layer->print_z < print_z; });
+		return (it == m_layers.end() || (*it)->print_z != print_z) ? nullptr : *it;
+	}
+    Layer*			get_layer_at_printz(coordf_t print_z) { return const_cast<Layer*>(std::as_const(*this).get_layer_at_printz(print_z)); }
+    // Get a layer approximately at print_z.
+    const Layer*	get_layer_at_printz(coordf_t print_z, coordf_t epsilon) const {
+        coordf_t limit = print_z - epsilon;
+        auto it = Slic3r::lower_bound_by_predicate(m_layers.begin(), m_layers.end(), [limit](const Layer *layer) { return layer->print_z < limit; });
+        return (it == m_layers.end() || (*it)->print_z > print_z + epsilon) ? nullptr : *it;
+	}
+    Layer*			get_layer_at_printz(coordf_t print_z, coordf_t epsilon) { return const_cast<Layer*>(std::as_const(*this).get_layer_at_printz(print_z, epsilon)); }
 
     // print_z: top of the layer; slice_z: center of the layer.
     Layer* add_layer(int id, coordf_t height, coordf_t print_z, coordf_t slice_z);
@@ -148,17 +194,16 @@ public:
     std::vector<ExPolygons>     slice_support_blockers() const { return this->slice_support_volumes(ModelVolumeType::SUPPORT_BLOCKER); }
     std::vector<ExPolygons>     slice_support_enforcers() const { return this->slice_support_volumes(ModelVolumeType::SUPPORT_ENFORCER); }
 
-protected:
+private:
     // to be called from Print only.
     friend class Print;
 
-	PrintObject(Print* print, ModelObject* model_object, bool add_instances = true);
-	~PrintObject() {}
+	PrintObject(Print* print, ModelObject* model_object, const Transform3d& trafo, PrintInstances&& instances);
+	~PrintObject() = default;
 
     void                    config_apply(const ConfigBase &other, bool ignore_nonexistent = false) { this->m_config.apply(other, ignore_nonexistent); }
     void                    config_apply_only(const ConfigBase &other, const t_config_option_keys &keys, bool ignore_nonexistent = false) { this->m_config.apply_only(other, keys, ignore_nonexistent); }
-    void                    set_trafo(const Transform3d& trafo) { m_trafo = trafo; }
-    PrintBase::ApplyStatus  set_copies(const Points &points);
+    PrintBase::ApplyStatus  set_instances(PrintInstances &&instances);
     // Invalidates the step, and its depending steps in PrintObject and Print.
     bool                    invalidate_step(PrintObjectStep step);
     // Invalidates all PrintObject and Print steps.
@@ -179,8 +224,7 @@ private:
 
     void _slice(const std::vector<coordf_t> &layer_height_profile);
     std::string _fix_slicing_errors();
-    void _simplify_slices(double distance);
-    void _make_perimeters();
+    void simplify_slices(double distance);
     bool has_support_material() const;
     void detect_surfaces_type();
     void process_external_surfaces();
@@ -191,25 +235,30 @@ private:
     void combine_infill();
     void _generate_support_material();
 
+    // XYZ in scaled coordinates
+    Vec3crd									m_size;
     PrintObjectConfig                       m_config;
     // Translation in Z + Rotation + Scaling / Mirroring.
     Transform3d                             m_trafo = Transform3d::Identity();
     // Slic3r::Point objects in scaled G-code coordinates
-    Points                                  m_copies;
-    // scaled coordinates to add to copies (to compensate for the alignment
-    // operated when creating the object but still preserving a coherent API
-    // for external callers)
-    Point                                   m_copies_shift;
+    std::vector<PrintInstance>              m_instances;
+    // The mesh is being centered before thrown to Clipper, so that the Clipper's fixed coordinates require less bits.
+    // This is the adjustment of the  the Object's coordinate system towards PrintObject's coordinate system.
+    Point                                   m_center_offset;
 
     SlicingParameters                       m_slicing_params;
     LayerPtrs                               m_layers;
     SupportLayerPtrs                        m_support_layers;
 
-    std::vector<ExPolygons> slice_region(size_t region_id, const std::vector<float> &z) const;
+    // this is set to true when LayerRegion->slices is split in top/internal/bottom
+    // so that next call to make_perimeters() performs a union() before computing loops
+    bool                    				m_typed_slices = false;
+
+    std::vector<ExPolygons> slice_region(size_t region_id, const std::vector<float> &z, SlicingMode mode) const;
     std::vector<ExPolygons> slice_modifiers(size_t region_id, const std::vector<float> &z) const;
-    std::vector<ExPolygons> slice_volumes(const std::vector<float> &z, const std::vector<const ModelVolume*> &volumes) const;
-    std::vector<ExPolygons> slice_volume(const std::vector<float> &z, const ModelVolume &volume) const;
-    std::vector<ExPolygons> slice_volume(const std::vector<float> &z, const std::vector<t_layer_height_range> &ranges, const ModelVolume &volume) const;
+    std::vector<ExPolygons> slice_volumes(const std::vector<float> &z, SlicingMode mode, const std::vector<const ModelVolume*> &volumes) const;
+    std::vector<ExPolygons> slice_volume(const std::vector<float> &z, SlicingMode mode, const ModelVolume &volume) const;
+    std::vector<ExPolygons> slice_volume(const std::vector<float> &z, const std::vector<t_layer_height_range> &ranges, SlicingMode mode, const ModelVolume &volume) const;
 };
 
 struct WipeTowerData
@@ -217,7 +266,7 @@ struct WipeTowerData
     // Following section will be consumed by the GCodeGenerator.
     // Tool ordering of a non-sequential print has to be known to calculate the wipe tower.
     // Cache it here, so it does not need to be recalculated during the G-code generation.
-    ToolOrdering                                          tool_ordering;
+    ToolOrdering                                         &tool_ordering;
     // Cache of tool changes per print layer.
     std::unique_ptr<std::vector<WipeTower::ToolChangeResult>> priming;
     std::vector<std::vector<WipeTower::ToolChangeResult>> tool_changes;
@@ -227,16 +276,25 @@ struct WipeTowerData
 
     // Depth of the wipe tower to pass to GLCanvas3D for exact bounding box:
     float                                                 depth;
+    float                                                 brim_width;
 
     void clear() {
-        tool_ordering.clear();
         priming.reset(nullptr);
         tool_changes.clear();
         final_purge.reset(nullptr);
         used_filament.clear();
         number_of_toolchanges = -1;
         depth = 0.f;
+        brim_width = 0.f;
     }
+
+private:
+	// Only allow the WipeTowerData to be instantiated internally by Print, 
+	// as this WipeTowerData shares reference to Print::m_tool_ordering.
+	friend class Print;
+	WipeTowerData(ToolOrdering &tool_ordering) : tool_ordering(tool_ordering) { clear(); }
+	WipeTowerData(const WipeTowerData & /* rhs */) = delete;
+	WipeTowerData &operator=(const WipeTowerData & /* rhs */) = delete;
 };
 
 struct PrintStatistics
@@ -244,11 +302,12 @@ struct PrintStatistics
     PrintStatistics() { clear(); }
     std::string                     estimated_normal_print_time;
     std::string                     estimated_silent_print_time;
-    std::vector<std::string>        estimated_normal_color_print_times;
-    std::vector<std::string>        estimated_silent_color_print_times;
+    std::vector<std::pair<CustomGcodeType, std::string>>    estimated_normal_custom_gcode_print_times;
+    std::vector<std::pair<CustomGcodeType, std::string>>    estimated_silent_custom_gcode_print_times;
     double                          total_used_filament;
     double                          total_extruded_volume;
     double                          total_cost;
+    int                             total_toolchanges;
     double                          total_weight;
     double                          total_wipe_tower_cost;
     double                          total_wipe_tower_filament;
@@ -264,11 +323,12 @@ struct PrintStatistics
     void clear() {
         estimated_normal_print_time.clear();
         estimated_silent_print_time.clear();
-        estimated_normal_color_print_times.clear();
-        estimated_silent_color_print_times.clear();
+        estimated_normal_custom_gcode_print_times.clear();
+        estimated_silent_custom_gcode_print_times.clear();
         total_used_filament    = 0.;
         total_extruded_volume  = 0.;
         total_cost             = 0.;
+        total_toolchanges      = 0;
         total_weight           = 0.;
         total_wipe_tower_cost  = 0.;
         total_wipe_tower_filament = 0.;
@@ -286,10 +346,10 @@ private: // Prevents erroneous use by other classes.
     typedef PrintBaseWithState<PrintStep, psCount> Inherited;
 
 public:
-    Print() {}
+    Print() = default;
 	virtual ~Print() { this->clear(); }
 
-	PrinterTechnology	technology() const noexcept { return ptFFF; }
+	PrinterTechnology	technology() const noexcept override { return ptFFF; }
 
     // Methods, which change the state of Print / PrintObject / PrintRegion.
     // The following methods are synchronized with process() and export_gcode(),
@@ -319,12 +379,9 @@ public:
 
     bool                has_infinite_skirt() const;
     bool                has_skirt() const;
-    float               get_wipe_tower_depth() const { return m_wipe_tower_data.depth; }
 
     // Returns an empty string if valid, otherwise returns an error message.
     std::string         validate() const override;
-    BoundingBox         bounding_box() const;
-    BoundingBox         total_bounding_box() const;
     double              skirt_first_layer_height() const;
     Flow                brim_flow() const;
     Flow                skirt_flow() const;
@@ -340,6 +397,7 @@ public:
     const PrintConfig&          config() const { return m_config; }
     const PrintObjectConfig&    default_object_config() const { return m_default_object_config; }
     const PrintRegionConfig&    default_region_config() const { return m_default_region_config; }
+    //FIXME returning const vector to non-const PrintObject*, caller could modify PrintObjects!
     const PrintObjectPtrs&      objects() const { return m_objects; }
     PrintObject*                get_object(size_t idx) { return m_objects[idx]; }
     const PrintObject*          get_object(size_t idx) const { return m_objects[idx]; }
@@ -348,9 +406,6 @@ public:
     // If zero, then the print is empty and the print shall not be executed.
     unsigned int                num_object_instances() const;
 
-    // Returns extruder this eec should be printed with, according to PrintRegion config:
-    static int                  get_extruder(const ExtrusionEntityCollection& fill, const PrintRegion &region);
-
     const ExtrusionEntityCollection& skirt() const { return m_skirt; }
     const ExtrusionEntityCollection& brim() const { return m_brim; }
 
@@ -358,12 +413,14 @@ public:
 
     // Wipe tower support.
     bool                        has_wipe_tower() const;
-    const WipeTowerData&        wipe_tower_data() const { return m_wipe_tower_data; }
+    const WipeTowerData&        wipe_tower_data(size_t extruders_cnt = 0, double first_layer_height = 0., double nozzle_diameter = 0.) const;
+    const ToolOrdering& 		tool_ordering() const { return m_tool_ordering; }
 
 	std::string                 output_filename(const std::string &filename_base = std::string()) const override;
 
     // Accessed by SupportMaterial
     const PrintRegion*  get_region(size_t idx) const  { return m_regions[idx]; }
+    const ToolOrdering& get_tool_ordering() const { return m_wipe_tower_data.tool_ordering; }   // #ys_FIXME just for testing
 
 protected:
     // methods for handling regions
@@ -379,7 +436,6 @@ private:
 		const DynamicPrintConfig &new_full_config, 
 		t_config_option_keys &print_diff, t_config_option_keys &object_diff, t_config_option_keys &region_diff, 
 		t_config_option_keys &full_config_diff, 
-		DynamicPrintConfig &placeholder_parser_overrides,
 		DynamicPrintConfig &filament_overrides) const;
 
     bool                invalidate_state_by_config_options(const std::vector<t_config_option_key> &opt_keys);
@@ -387,7 +443,6 @@ private:
     void                _make_skirt();
     void                _make_brim();
     void                _make_wipe_tower();
-    void                _simplify_slices(double distance);
 
     // Declared here to have access to Model / ModelObject / ModelInstance
     static void         model_volume_list_update_supports(ModelObject &model_object_dst, const ModelObject &model_object_src);
@@ -403,7 +458,8 @@ private:
     ExtrusionEntityCollection               m_brim;
 
     // Following section will be consumed by the GCodeGenerator.
-    WipeTowerData                           m_wipe_tower_data;
+    ToolOrdering 							m_tool_ordering;
+    WipeTowerData                           m_wipe_tower_data {m_tool_ordering};
 
     // Estimated print time, filament consumed.
     PrintStatistics                         m_print_statistics;
