@@ -66,6 +66,10 @@ bool Version::is_current_slic3r_supported() const
 	return this->is_slic3r_supported(Slic3r::SEMVER);
 }
 
+bool Version::is_current_slic3r_downgrade() const
+{
+	return Slic3r::SEMVER < min_slic3r_version;
+}
 #if 0
 //TODO: This test should be moved to a unit test, once we have C++ unit tests in place.
 static int version_test()
@@ -205,7 +209,7 @@ size_t Index::load(const boost::filesystem::path &path)
 #endif
     	++ idx_line;
     	// Skip the initial white spaces.
-    	char *key = left_trim(const_cast<char*>(line.data()));
+    	char *key = left_trim(line.data());
 		if (*key == '#')
 			// Skip a comment line.
 			continue;
@@ -227,17 +231,17 @@ size_t Index::load(const boost::filesystem::path &path)
 				// End of semver or keyword.
 				break;
     	}
-    	if (*key_end != 0 && *key_end != ' ' && *key_end != '\t' && *key_end != '=')
+        if (*key_end != 0 && *key_end != ' ' && *key_end != '\t' && *key_end != '=')
     		throw file_parser_error("Invalid keyword or semantic version", path, idx_line);
-		char *value = left_trim(key_end);
+        char *value = left_trim(key_end);
 		bool  key_value_pair = *value == '=';
 		if (key_value_pair)
 			value = left_trim(value + 1);
 		*key_end = 0;
     	boost::optional<Semver> semver;
-    	if (maybe_semver)
+        if (maybe_semver)
     		semver = Semver::parse(key);
-		if (key_value_pair) {
+        if (key_value_pair) {
     		if (semver)
     			throw file_parser_error("Key cannot be a semantic version", path, idx_line);\
     		// Verify validity of the key / value pair.
@@ -245,11 +249,11 @@ size_t Index::load(const boost::filesystem::path &path)
     		if (strcmp(key, "min_slic3r_version") == 0 || strcmp(key, "max_slic3r_version") == 0) {
     			if (! svalue.empty())
 					semver = Semver::parse(svalue);
-		    	if (! semver)
+                if (! semver)
 		    		throw file_parser_error(std::string(key) + " must referece a valid semantic version", path, idx_line);
-				if (strcmp(key, "min_slic3r_version") == 0)
+                if (strcmp(key, "min_slic3r_version") == 0)
     				ver.min_slic3r_version = *semver;
-    			else
+                else
     				ver.max_slic3r_version = *semver;
     		} else {
     			// Ignore unknown keys, as there may come new keys in the future.
@@ -288,7 +292,6 @@ Index::const_iterator Index::find(const Semver &ver) const
 
 Index::const_iterator Index::recommended(const Semver &slic3r_version) const
 {
-	int idx = -1;
 	const_iterator highest = this->end();
 	for (const_iterator it = this->begin(); it != this->end(); ++ it)
 		if (it->is_slic3r_supported(slic3r_version) &&
