@@ -303,6 +303,8 @@ public:
         int             instance_id;
 		bool operator==(const CompositeID &rhs) const { return object_id == rhs.object_id && volume_id == rhs.volume_id && instance_id == rhs.instance_id; }
 		bool operator!=(const CompositeID &rhs) const { return ! (*this == rhs); }
+		bool operator< (const CompositeID &rhs) const 
+			{ return object_id < rhs.object_id || (object_id == rhs.object_id && (volume_id < rhs.volume_id || (volume_id == rhs.volume_id && instance_id < rhs.instance_id))); }
     };
     CompositeID         composite_id;
     // Fingerprint of the source geometry. For ModelVolumes, it is the ModelVolume::ID and ModelInstanceID, 
@@ -311,33 +313,38 @@ public:
     // Valid geometry_id should always be positive.
     std::pair<size_t, size_t> geometry_id;
     // An ID containing the extruder ID (used to select color).
-    int                 extruder_id;
-    // Is this object selected?
-    bool                selected;
-    // Is this object disabled from selection?
-    bool                disabled;
-    // Is this object printable?
-    bool                printable;
-    // Whether or not this volume is active for rendering
-    bool                is_active;
-    // Whether or not to use this volume when applying zoom_to_volumes()
-    bool                zoom_to_volumes;
-    // Wheter or not this volume is enabled for outside print volume detection in shader.
-    bool                shader_outside_printer_detection_enabled;
-    // Wheter or not this volume is outside print volume.
-    bool                is_outside;
+    int                 	extruder_id;
+
+    // Various boolean flags.
+    struct {
+	    // Is this object selected?
+	    bool                selected : 1;
+	    // Is this object disabled from selection?
+	    bool                disabled : 1;
+	    // Is this object printable?
+	    bool                printable : 1;
+	    // Whether or not this volume is active for rendering
+	    bool                is_active : 1;
+	    // Whether or not to use this volume when applying zoom_to_volumes()
+	    bool                zoom_to_volumes : 1;
+	    // Wheter or not this volume is enabled for outside print volume detection in shader.
+	    bool                shader_outside_printer_detection_enabled : 1;
+	    // Wheter or not this volume is outside print volume.
+	    bool                is_outside : 1;
+	    // Wheter or not this volume has been generated from a modifier
+	    bool                is_modifier : 1;
+	    // Wheter or not this volume has been generated from the wipe tower
+	    bool                is_wipe_tower : 1;
+	    // Wheter or not this volume has been generated from an extrusion path
+	    bool                is_extrusion_path : 1;
+	    // Wheter or not to always render this volume using its own alpha 
+	    bool                force_transparent : 1;
+	    // Whether or not always use the volume's own color (not using SELECTED/HOVER/DISABLED/OUTSIDE)
+	    bool                force_native_color : 1;
+	};
+
     // Is mouse or rectangle selection over this object to select/deselect it ?
-    EHoverState         hover;
-    // Wheter or not this volume has been generated from a modifier
-    bool                is_modifier;
-    // Wheter or not this volume has been generated from the wipe tower
-    bool                is_wipe_tower;
-    // Wheter or not this volume has been generated from an extrusion path
-    bool                is_extrusion_path;
-    // Wheter or not to always render this volume using its own alpha 
-    bool                force_transparent;
-    // Whether or not always use the volume's own color (not using SELECTED/HOVER/DISABLED/OUTSIDE)
-    bool                force_native_color;
+    EHoverState         	hover;
 
     // Interleaved triangles & normals with indexed triangles & quads.
     GLIndexedVertexArray        indexed_vertex_array;
@@ -656,6 +663,7 @@ public:
 
     static void thick_lines_to_verts(const Lines& lines, const std::vector<double>& widths, const std::vector<double>& heights, bool closed, double top_z, GLVolume& volume);
     static void thick_lines_to_verts(const Lines3& lines, const std::vector<double>& widths, const std::vector<double>& heights, bool closed, GLVolume& volume);
+	static void extrusionentity_to_verts(const Polyline &polyline, float width, float height, float print_z, GLVolume& volume);
     static void extrusionentity_to_verts(const ExtrusionPath& extrusion_path, float print_z, GLVolume& volume);
     static void extrusionentity_to_verts(const ExtrusionPath& extrusion_path, float print_z, const Point& copy, GLVolume& volume);
     static void extrusionentity_to_verts(const ExtrusionLoop& extrusion_loop, float print_z, const Point& copy, GLVolume& volume);
