@@ -1,5 +1,6 @@
 #include "BackgroundSlicingProcess.hpp"
 #include "GUI_App.hpp"
+#include "GUI.hpp"
 
 #include <wx/app.h>
 #include <wx/panel.h>
@@ -33,6 +34,8 @@
 #include <boost/log/trivial.hpp>
 #include <boost/nowide/cstdio.hpp>
 #include "I18N.hpp"
+#include "GUI.hpp"
+#include "RemovableDriveManager.hpp"
 
 namespace Slic3r {
 
@@ -59,6 +62,7 @@ bool BackgroundSlicingProcess::select_technology(PrinterTechnology tech)
 		switch (tech) {
 		case ptFFF: m_print = m_fff_print; break;
 		case ptSLA: m_print = m_sla_print; break;
+        default: assert(false); break;
 		}
 		changed = true;
 	}
@@ -91,13 +95,35 @@ void BackgroundSlicingProcess::process_fff()
 #else
     m_fff_print->export_gcode(m_temp_output_path, m_gcode_preview_data);
 #endif // ENABLE_THUMBNAIL_GENERATOR
-    if (this->set_step_started(bspsGCodeFinalize)) {
+
+	if (this->set_step_started(bspsGCodeFinalize)) {
 	    if (! m_export_path.empty()) {
 	    	//FIXME localize the messages
 	    	// Perform the final post-processing of the export path by applying the print statistics over the file name.
 	    	std::string export_path = m_fff_print->print_statistics().finalize_output_path(m_export_path);
-		    if (copy_file(m_temp_output_path, export_path) != 0)
-	    		throw std::runtime_error(_utf8(L("Copying of the temporary G-code to the output G-code failed. Maybe the SD card is write locked?")));
+			int copy_ret_val = copy_file(m_temp_output_path, export_path, m_export_path_on_removable_media);
+			switch (copy_ret_val) {
+			case SUCCESS: break; // no error
+			case FAIL_COPY_FILE:
+				throw std::runtime_error(_utf8(L("Copying of the temporary G-code to the output G-code failed. Maybe the SD card is write locked?")));
+				break;
+			case FAIL_FILES_DIFFERENT: 
+				throw std::runtime_error((boost::format(_utf8(L("Copying of the temporary G-code to the output G-code failed. There might be problem with target device, please try exporting again or using different device. The corrupted output G-code is at %1%.tmp."))) % export_path).str());
+				break;
+			case FAIL_RENAMING: 
+				throw std::runtime_error((boost::format(_utf8(L("Renaming of the G-code after copying to the selected destination folder has failed. Current path is %1%.tmp. Please try exporting again."))) % export_path).str()); 
+				break;
+			case FAIL_CHECK_ORIGIN_NOT_OPENED: 
+				throw std::runtime_error((boost::format(_utf8(L("Copying of the temporary G-code has finished but the original code at %1% couldn't be opened during copy check. The output G-code is at %2%.tmp."))) % m_temp_output_path % export_path).str());
+				break;
+			case FAIL_CHECK_TARGET_NOT_OPENED: 
+				throw std::runtime_error((boost::format(_utf8(L("Copying of the temporary G-code has finished but the exported code couldn't be opened during copy check. The output G-code is at %1%.tmp."))) % export_path).str()); 
+				break;
+			default:
+				BOOST_LOG_TRIVIAL(warning) << "Unexpected fail code(" << (int)copy_ret_val << ") durring copy_file() to " << export_path << ".";
+				break;
+			}
+			
 	    	m_print->set_status(95, _utf8(L("Running post-processing scripts")));
 	    	run_post_process_scripts(export_path, m_fff_print->config());
 	    	m_print->set_status(100, (boost::format(_utf8(L("G-code file exported to %1%"))) % export_path).str());
@@ -110,14 +136,46 @@ void BackgroundSlicingProcess::process_fff()
 	}
 }
 
+#if ENABLE_THUMBNAIL_GENERATOR
+static void write_thumbnail(Zipper& zipper, const ThumbnailData& data)
+{
+    size_t png_size = 0;
+    void* png_data = tdefl_write_image_to_png_file_in_memory_ex((const void*)data.pixels.data(), data.width, data.height, 4, &png_size, MZ_DEFAULT_LEVEL, 1);
+    if (png_data != nullptr)
+    {
+        zipper.add_entry("thumbnail/thumbnail" + std::to_string(data.width) + "x" + std::to_string(data.height) + ".png", (const std::uint8_t*)png_data, png_size);
+        mz_free(png_data);
+    }
+}
+#endif // ENABLE_THUMBNAIL_GENERATOR
+
 void BackgroundSlicingProcess::process_sla()
 {
     assert(m_print == m_sla_print);
     m_print->process();
     if (this->set_step_started(bspsGCodeFinalize)) {
         if (! m_export_path.empty()) {
-        	const std::string export_path = m_sla_print->print_statistics().finalize_output_path(m_export_path);
-            m_sla_print->export_raster(export_path);
+            const std::string export_path = m_sla_print->print_statistics().finalize_output_path(m_export_path);
+
+            Zipper zipper(export_path);
+            m_sla_print->export_raster(zipper);
+
+#if ENABLE_THUMBNAIL_GENERATOR
+            if (m_thumbnail_cb != nullptr)
+            {
+                ThumbnailsList thumbnails;
+                m_thumbnail_cb(thumbnails, current_print()->full_print_config().option<ConfigOptionPoints>("thumbnails")->values, true, true, true, true);
+//                m_thumbnail_cb(thumbnails, current_print()->full_print_config().option<ConfigOptionPoints>("thumbnails")->values, true, false, true, true); // renders also supports and pad
+                for (const ThumbnailData& data : thumbnails)
+                {
+                    if (data.is_valid())
+                        write_thumbnail(zipper, data);
+                }
+            }
+#endif // ENABLE_THUMBNAIL_GENERATOR
+
+            zipper.finalize();
+
             m_print->set_status(100, (boost::format(_utf8(L("Masked SLA file exported to %1%"))) % export_path).str());
         } else if (! m_upload_job.empty()) {
             prepare_upload();
@@ -160,10 +218,10 @@ void BackgroundSlicingProcess::thread_proc()
 			// Canceled, this is all right.
 			assert(m_print->canceled());
         } catch (const std::bad_alloc& ex) {
-            wxString errmsg = wxString::Format(_(L("%s has encountered an error. It was likely caused by running out of memory. "
+            wxString errmsg = GUI::from_u8((boost::format(_utf8(L("%s has encountered an error. It was likely caused by running out of memory. "
                                   "If you are sure you have enough RAM on your system, this may also be a bug and we would "
-                                  "be glad if you reported it.")), SLIC3R_APP_NAME);
-            error = errmsg.ToStdString() + "\n\n" + std::string(ex.what());
+                                  "be glad if you reported it."))) % SLIC3R_APP_NAME).str());
+            error = std::string(errmsg.ToUTF8()) + "\n\n" + std::string(ex.what());
         } catch (std::exception &ex) {
 			error = ex.what();
 		} catch (...) {
@@ -176,7 +234,7 @@ void BackgroundSlicingProcess::thread_proc()
 			// Only post the canceled event, if canceled by user.
 			// Don't post the canceled event, if canceled from Print::apply().
 			wxCommandEvent evt(m_event_finished_id);
-			evt.SetString(error);
+            evt.SetString(GUI::from_u8(error));
 			evt.SetInt(m_print->canceled() ? -1 : (error.empty() ? 1 : 0));
         	wxQueueEvent(GUI::wxGetApp().mainframe->m_plater, evt.Clone());
         }
@@ -228,11 +286,7 @@ bool BackgroundSlicingProcess::start()
 	if (m_state == STATE_INITIAL) {
 		// The worker thread is not running yet. Start it.
 		assert(! m_thread.joinable());
-		boost::thread::attributes attrs;
-		// Duplicating the stack allocation size of Thread Building Block worker threads of the thread pool:
-		// allocate 4MB on a 64bit system, allocate 2MB on a 32bit system by default.
-		attrs.set_stack_size((sizeof(void*) == 4) ? (2048 * 1024) : (4096 * 1024));
-		m_thread = boost::thread(attrs, [this]{this->thread_proc_safe();});
+		m_thread = create_thread([this]{this->thread_proc_safe();});
 		// Wait until the worker thread is ready to execute the background processing task.
 		m_condition.wait(lck, [this](){ return m_state == STATE_IDLE; });
 	}
@@ -347,7 +401,7 @@ void BackgroundSlicingProcess::set_task(const PrintBase::TaskParams &params)
 }
 
 // Set the output path of the G-code.
-void BackgroundSlicingProcess::schedule_export(const std::string &path)
+void BackgroundSlicingProcess::schedule_export(const std::string &path, bool export_path_on_removable_media)
 { 
 	assert(m_export_path.empty());
 	if (! m_export_path.empty())
@@ -357,6 +411,7 @@ void BackgroundSlicingProcess::schedule_export(const std::string &path)
 	tbb::mutex::scoped_lock lock(m_print->state_mutex());
 	this->invalidate_step(bspsGCodeFinalize);
 	m_export_path = path;
+	m_export_path_on_removable_media = export_path_on_removable_media;
 }
 
 void BackgroundSlicingProcess::schedule_upload(Slic3r::PrintHostJob upload_job)
@@ -377,6 +432,7 @@ void BackgroundSlicingProcess::reset_export()
 	assert(! this->running());
 	if (! this->running()) {
 		m_export_path.clear();
+		m_export_path_on_removable_media = false;
 		// invalidate_step expects the mutex to be locked.
 		tbb::mutex::scoped_lock lock(m_print->state_mutex());
 		this->invalidate_step(bspsGCodeFinalize);
@@ -421,17 +477,33 @@ void BackgroundSlicingProcess::prepare_upload()
 
 	if (m_print == m_fff_print) {
 		m_print->set_status(95, _utf8(L("Running post-processing scripts")));
-		if (copy_file(m_temp_output_path, source_path.string()) != 0) {
+		if (copy_file(m_temp_output_path, source_path.string()) != SUCCESS) {
 			throw std::runtime_error(_utf8(L("Copying of the temporary G-code to the output G-code failed")));
 		}
 		run_post_process_scripts(source_path.string(), m_fff_print->config());
-		m_upload_job.upload_data.upload_path = m_fff_print->print_statistics().finalize_output_path(m_upload_job.upload_data.upload_path.string());
+        m_upload_job.upload_data.upload_path = m_fff_print->print_statistics().finalize_output_path(m_upload_job.upload_data.upload_path.string());
     } else {
-		m_upload_job.upload_data.upload_path = m_sla_print->print_statistics().finalize_output_path(m_upload_job.upload_data.upload_path.string());
-        m_sla_print->export_raster(source_path.string(), m_upload_job.upload_data.upload_path.string());
-	}
+        m_upload_job.upload_data.upload_path = m_sla_print->print_statistics().finalize_output_path(m_upload_job.upload_data.upload_path.string());
 
-	m_print->set_status(100, (boost::format(_utf8(L("Scheduling upload to `%1%`. See Window -> Print Host Upload Queue"))) % m_upload_job.printhost->get_host()).str());
+        Zipper zipper{source_path.string()};
+        m_sla_print->export_raster(zipper, m_upload_job.upload_data.upload_path.string());
+#if ENABLE_THUMBNAIL_GENERATOR
+        if (m_thumbnail_cb != nullptr)
+        {
+            ThumbnailsList thumbnails;
+            m_thumbnail_cb(thumbnails, current_print()->full_print_config().option<ConfigOptionPoints>("thumbnails")->values, true, true, true, true);
+//            m_thumbnail_cb(thumbnails, current_print()->full_print_config().option<ConfigOptionPoints>("thumbnails")->values, true, false, true, true); // renders also supports and pad
+            for (const ThumbnailData& data : thumbnails)
+            {
+                if (data.is_valid())
+                    write_thumbnail(zipper, data);
+            }
+        }
+#endif // ENABLE_THUMBNAIL_GENERATOR
+        zipper.finalize();
+    }
+
+    m_print->set_status(100, (boost::format(_utf8(L("Scheduling upload to `%1%`. See Window -> Print Host Upload Queue"))) % m_upload_job.printhost->get_host()).str());
 
 	m_upload_job.upload_data.source_path = std::move(source_path);
 
