@@ -6,11 +6,12 @@
 #endif // !ENABLE_THUMBNAIL_GENERATOR
 #include "GUI_App.hpp"
 #include "AppConfig.hpp"
+#if ENABLE_CAMERA_STATISTICS
+#include "Mouse3DController.hpp"
+#endif // ENABLE_CAMERA_STATISTICS
 
 #include <GL/glew.h>
 
-static const float GIMBALL_LOCK_THETA_MAX = 180.0f;
-
 // phi / theta angles to orient the camera.
 static const float VIEW_DEFAULT[2] = { 45.0f, 45.0f };
 static const float VIEW_LEFT[2] = { 90.0f, 90.0f };
@@ -34,31 +35,28 @@ double Camera::FrustrumZMargin = 10.0;
 double Camera::MaxFovDeg = 60.0;
 
 Camera::Camera()
-    : phi(45.0f)
-    , requires_zoom_to_bed(false)
-    , inverted_phi(false)
+    : requires_zoom_to_bed(false)
     , m_type(Perspective)
     , m_target(Vec3d::Zero())
-    , m_theta(45.0f)
+    , m_zenit(45.0f)
     , m_zoom(1.0)
     , m_distance(DefaultDistance)
     , m_gui_scale(1.0)
     , m_view_matrix(Transform3d::Identity())
+    , m_view_rotation(1., 0., 0., 0.)
     , m_projection_matrix(Transform3d::Identity())
 {
+    set_default_orientation();
 }
 
 std::string Camera::get_type_as_string() const
 {
     switch (m_type)
     {
-    case Unknown:
-        return "unknown";
-    case Perspective:
-        return "perspective";
+    case Unknown:     return "unknown";
+    case Perspective: return "perspective";
     default:
-    case Ortho:
-        return "orthographic";
+    case Ortho:       return "orthographic";
     };
 }
 
@@ -74,10 +72,7 @@ void Camera::set_type(EType type)
 
 void Camera::set_type(const std::string& type)
 {
-    if (type == "1")
-        set_type(Perspective);
-    else
-        set_type(Ortho);
+    set_type((type == "1") ? Perspective : Ortho);
 }
 
 void Camera::select_next_type()
@@ -91,67 +86,47 @@ void Camera::select_next_type()
 
 void Camera::set_target(const Vec3d& target)
 {
-    m_target = target;
-    m_target(0) = clamp(m_scene_box.min(0), m_scene_box.max(0), m_target(0));
-    m_target(1) = clamp(m_scene_box.min(1), m_scene_box.max(1), m_target(1));
-    m_target(2) = clamp(m_scene_box.min(2), m_scene_box.max(2), m_target(2));
-}
-
-void Camera::set_theta(float theta, bool apply_limit)
-{
-    if (apply_limit)
-        m_theta = clamp(0.0f, GIMBALL_LOCK_THETA_MAX, theta);
-    else
+    Vec3d new_target = validate_target(target);
+    Vec3d new_displacement = new_target - m_target;
+    if (!new_displacement.isApprox(Vec3d::Zero()))
     {
-        m_theta = fmod(theta, 360.0f);
-        if (m_theta < 0.0f)
-            m_theta += 360.0f;
+        m_target = new_target;
+        m_view_matrix.translate(-new_displacement);
     }
 }
 
-void Camera::set_zoom(double zoom, const BoundingBoxf3& max_box, int canvas_w, int canvas_h)
+void Camera::update_zoom(double delta_zoom)
 {
-    zoom = std::max(std::min(zoom, 4.0), -4.0) / 10.0;
-    zoom = m_zoom / (1.0 - zoom);
+    set_zoom(m_zoom / (1.0 - std::max(std::min(delta_zoom, 4.0), -4.0) * 0.1));
+}
 
+void Camera::set_zoom(double zoom)
+{
     // Don't allow to zoom too far outside the scene.
-    double zoom_min = calc_zoom_to_bounding_box_factor(max_box, canvas_w, canvas_h);
+    double zoom_min = min_zoom();
     if (zoom_min > 0.0)
-        zoom = std::max(zoom, zoom_min * 0.7);
+        zoom = std::max(zoom, zoom_min);
 
     // Don't allow to zoom too close to the scene.
-    zoom = std::min(zoom, 100.0);
-
-    m_zoom = zoom;
+    m_zoom = std::min(zoom, max_zoom());
 }
 
-bool Camera::select_view(const std::string& direction)
+void Camera::select_view(const std::string& direction)
 {
-    const float* dir_vec = nullptr;
-
     if (direction == "iso")
-        dir_vec = VIEW_DEFAULT;
+        set_default_orientation();
     else if (direction == "left")
-        dir_vec = VIEW_LEFT;
+        look_at(m_target - m_distance * Vec3d::UnitX(), m_target, Vec3d::UnitZ());
     else if (direction == "right")
-        dir_vec = VIEW_RIGHT;
+        look_at(m_target + m_distance * Vec3d::UnitX(), m_target, Vec3d::UnitZ());
     else if (direction == "top")
-        dir_vec = VIEW_TOP;
+        look_at(m_target + m_distance * Vec3d::UnitZ(), m_target, Vec3d::UnitY());
     else if (direction == "bottom")
-        dir_vec = VIEW_BOTTOM;
+        look_at(m_target - m_distance * Vec3d::UnitZ(), m_target, -Vec3d::UnitY());
     else if (direction == "front")
-        dir_vec = VIEW_FRONT;
+        look_at(m_target - m_distance * Vec3d::UnitY(), m_target, Vec3d::UnitZ());
     else if (direction == "rear")
-        dir_vec = VIEW_REAR;
-
-    if (dir_vec != nullptr)
-    {
-        phi = dir_vec[0];
-        set_theta(dir_vec[1], false);
-        return true;
-    }
-    else
-        return false;
+        look_at(m_target + m_distance * Vec3d::UnitY(), m_target, Vec3d::UnitZ());
 }
 
 double Camera::get_fov() const
@@ -174,86 +149,52 @@ void Camera::apply_viewport(int x, int y, unsigned int w, unsigned int h) const
 
 void Camera::apply_view_matrix() const
 {
-    double theta_rad = Geometry::deg2rad(-(double)m_theta);
-    double phi_rad = Geometry::deg2rad((double)phi);
-    double sin_theta = ::sin(theta_rad);
-    Vec3d camera_pos = m_target + m_distance * Vec3d(sin_theta * ::sin(phi_rad), sin_theta * ::cos(phi_rad), ::cos(theta_rad));
-
     glsafe(::glMatrixMode(GL_MODELVIEW));
     glsafe(::glLoadIdentity());
-
-    glsafe(::glRotatef(-m_theta, 1.0f, 0.0f, 0.0f)); // pitch
-    glsafe(::glRotatef(phi, 0.0f, 0.0f, 1.0f));      // yaw
-
-    glsafe(::glTranslated(-camera_pos(0), -camera_pos(1), -camera_pos(2))); 
-
-    glsafe(::glGetDoublev(GL_MODELVIEW_MATRIX, m_view_matrix.data()));
+    glsafe(::glMultMatrixd(m_view_matrix.data()));
 }
 
 void Camera::apply_projection(const BoundingBoxf3& box, double near_z, double far_z) const
 {
-    set_distance(DefaultDistance);
-
     double w = 0.0;
     double h = 0.0;
 
-    while (true)
+    double old_distance = m_distance;
+    m_frustrum_zs = calc_tight_frustrum_zs_around(box);
+    if (m_distance != old_distance)
+        // the camera has been moved re-apply view matrix
+        apply_view_matrix();
+
+    if (near_z > 0.0)
+        m_frustrum_zs.first = std::max(std::min(m_frustrum_zs.first, near_z), FrustrumMinNearZ);
+
+    if (far_z > 0.0)
+        m_frustrum_zs.second = std::max(m_frustrum_zs.second, far_z);
+
+    w = 0.5 * (double)m_viewport[2];
+    h = 0.5 * (double)m_viewport[3];
+
+    double inv_zoom = get_inv_zoom();
+    w *= inv_zoom;
+    h *= inv_zoom;
+
+    switch (m_type)
     {
-        m_frustrum_zs = calc_tight_frustrum_zs_around(box);
-
-        if (near_z > 0.0)
-            m_frustrum_zs.first = std::min(m_frustrum_zs.first, near_z);
-
-        if (far_z > 0.0)
-            m_frustrum_zs.second = std::max(m_frustrum_zs.second, far_z);
-
-        w = 0.5 * (double)m_viewport[2];
-        h = 0.5 * (double)m_viewport[3];
-
-        if (m_zoom != 0.0)
-        {
-            double inv_zoom = 1.0 / m_zoom;
-            w *= inv_zoom;
-            h *= inv_zoom;
-        }
-
-        switch (m_type)
-        {
-        default:
-        case Ortho:
-        {
-            m_gui_scale = 1.0;
-            break;
-        }
-        case Perspective:
-        {
-            // scale near plane to keep w and h constant on the plane at z = m_distance
-            double scale = m_frustrum_zs.first / m_distance;
-            w *= scale;
-            h *= scale;
-            m_gui_scale = scale;
-            break;
-        }
-        }
-
-        if (m_type == Perspective)
-        {
-            double fov_deg = Geometry::rad2deg(2.0 * std::atan(h / m_frustrum_zs.first));
-
-            // adjust camera distance to keep fov in a limited range
-            if (fov_deg > MaxFovDeg)
-            {
-                double delta_z = h / ::tan(0.5 * Geometry::deg2rad(MaxFovDeg)) - m_frustrum_zs.first;
-                if (delta_z > 0.001)
-                    set_distance(m_distance + delta_z);
-                else
-                    break;
-            }
-            else
-                break;
-        }
-        else
-            break;
+    default:
+    case Ortho:
+    {
+        m_gui_scale = 1.0;
+        break;
+    }
+    case Perspective:
+    {
+        // scale near plane to keep w and h constant on the plane at z = m_distance
+        double scale = m_frustrum_zs.first / m_distance;
+        w *= scale;
+        h *= scale;
+        m_gui_scale = scale;
+        break;
+    }
     }
 
     glsafe(::glMatrixMode(GL_PROJECTION));
@@ -279,14 +220,14 @@ void Camera::apply_projection(const BoundingBoxf3& box, double near_z, double fa
 }
 
 #if ENABLE_THUMBNAIL_GENERATOR
-void Camera::zoom_to_box(const BoundingBoxf3& box, int canvas_w, int canvas_h, double margin_factor)
+void Camera::zoom_to_box(const BoundingBoxf3& box, double margin_factor)
 #else
 void Camera::zoom_to_box(const BoundingBoxf3& box, int canvas_w, int canvas_h)
 #endif // ENABLE_THUMBNAIL_GENERATOR
 {
     // Calculate the zoom factor needed to adjust the view around the given box.
 #if ENABLE_THUMBNAIL_GENERATOR
-    double zoom = calc_zoom_to_bounding_box_factor(box, canvas_w, canvas_h, margin_factor);
+    double zoom = calc_zoom_to_bounding_box_factor(box, margin_factor);
 #else
     double zoom = calc_zoom_to_bounding_box_factor(box, canvas_w, canvas_h);
 #endif // ENABLE_THUMBNAIL_GENERATOR
@@ -294,20 +235,20 @@ void Camera::zoom_to_box(const BoundingBoxf3& box, int canvas_w, int canvas_h)
     {
         m_zoom = zoom;
         // center view around box center
-        m_target = box.center();
+        set_target(box.center());
     }
 }
 
 #if ENABLE_THUMBNAIL_GENERATOR
-void Camera::zoom_to_volumes(const GLVolumePtrs& volumes, int canvas_w, int canvas_h, double margin_factor)
+void Camera::zoom_to_volumes(const GLVolumePtrs& volumes, double margin_factor)
 {
     Vec3d center;
-    double zoom = calc_zoom_to_volumes_factor(volumes, canvas_w, canvas_h, center, margin_factor);
+    double zoom = calc_zoom_to_volumes_factor(volumes, center, margin_factor);
     if (zoom > 0.0)
     {
         m_zoom = zoom;
         // center view around the calculated center
-        m_target = center;
+        set_target(center);
     }
 }
 #endif // ENABLE_THUMBNAIL_GENERATOR
@@ -316,13 +257,18 @@ void Camera::zoom_to_volumes(const GLVolumePtrs& volumes, int canvas_w, int canv
 void Camera::debug_render() const
 {
     ImGuiWrapper& imgui = *wxGetApp().imgui();
-    imgui.set_next_window_bg_alpha(0.5f);
     imgui.begin(std::string("Camera statistics"), ImGuiWindowFlags_AlwaysAutoResize | ImGuiWindowFlags_NoResize | ImGuiWindowFlags_NoCollapse);
 
     std::string type = get_type_as_string();
+    if (wxGetApp().plater()->get_mouse3d_controller().connected() || (wxGetApp().app_config->get("use_free_camera") == "1"))
+        type += "/free";
+    else
+        type += "/constrained";
+
     Vec3f position = get_position().cast<float>();
     Vec3f target = m_target.cast<float>();
     float distance = (float)get_distance();
+    float zenit = (float)m_zenit;
     Vec3f forward = get_dir_forward().cast<float>();
     Vec3f right = get_dir_right().cast<float>();
     Vec3f up = get_dir_up().cast<float>();
@@ -331,14 +277,17 @@ void Camera::debug_render() const
     float deltaZ = farZ - nearZ;
     float zoom = (float)m_zoom;
     float fov = (float)get_fov();
+    std::array<int, 4>viewport = get_viewport();
     float gui_scale = (float)get_gui_scale();
 
-    ImGui::InputText("Type", const_cast<char*>(type.data()), type.length(), ImGuiInputTextFlags_ReadOnly);
+    ImGui::InputText("Type", type.data(), type.length(), ImGuiInputTextFlags_ReadOnly);
     ImGui::Separator();
     ImGui::InputFloat3("Position", position.data(), "%.6f", ImGuiInputTextFlags_ReadOnly);
     ImGui::InputFloat3("Target", target.data(), "%.6f", ImGuiInputTextFlags_ReadOnly);
     ImGui::InputFloat("Distance", &distance, 0.0f, 0.0f, "%.6f", ImGuiInputTextFlags_ReadOnly);
     ImGui::Separator();
+    ImGui::InputFloat("Zenit", &zenit, 0.0f, 0.0f, "%.6f", ImGuiInputTextFlags_ReadOnly);
+    ImGui::Separator();
     ImGui::InputFloat3("Forward", forward.data(), "%.6f", ImGuiInputTextFlags_ReadOnly);
     ImGui::InputFloat3("Right", right.data(), "%.6f", ImGuiInputTextFlags_ReadOnly);
     ImGui::InputFloat3("Up", up.data(), "%.6f", ImGuiInputTextFlags_ReadOnly);
@@ -350,64 +299,96 @@ void Camera::debug_render() const
     ImGui::InputFloat("Zoom", &zoom, 0.0f, 0.0f, "%.6f", ImGuiInputTextFlags_ReadOnly);
     ImGui::InputFloat("Fov", &fov, 0.0f, 0.0f, "%.6f", ImGuiInputTextFlags_ReadOnly);
     ImGui::Separator();
+    ImGui::InputInt4("Viewport", viewport.data(), ImGuiInputTextFlags_ReadOnly);
+    ImGui::Separator();
     ImGui::InputFloat("GUI scale", &gui_scale, 0.0f, 0.0f, "%.6f", ImGuiInputTextFlags_ReadOnly);
     imgui.end();
 }
 #endif // ENABLE_CAMERA_STATISTICS
 
+void Camera::rotate_on_sphere(double delta_azimut_rad, double delta_zenit_rad, bool apply_limits)
+{
+    m_zenit += Geometry::rad2deg(delta_zenit_rad);
+    if (apply_limits) {
+        if (m_zenit > 90.0f) {
+            delta_zenit_rad -= Geometry::deg2rad(m_zenit - 90.0f);
+            m_zenit = 90.0f;
+        }
+        else if (m_zenit < -90.0f) {
+            delta_zenit_rad -= Geometry::deg2rad(m_zenit + 90.0f);
+            m_zenit = -90.0f;
+        }
+    }
+
+    Vec3d translation = m_view_matrix.translation() + m_view_rotation * m_target;
+    auto rot_z = Eigen::AngleAxisd(delta_azimut_rad, Vec3d::UnitZ());
+    m_view_rotation *= rot_z * Eigen::AngleAxisd(delta_zenit_rad, rot_z.inverse() * get_dir_right());
+    m_view_rotation.normalize();
+    m_view_matrix.fromPositionOrientationScale(m_view_rotation * (- m_target) + translation, m_view_rotation, Vec3d(1., 1., 1.));
+}
+
+// Virtual trackball, rotate around an axis, where the eucledian norm of the axis gives the rotation angle in radians.
+void Camera::rotate_local_around_target(const Vec3d& rotation_rad)
+{
+    double angle = rotation_rad.norm();
+    if (std::abs(angle) > EPSILON) {
+	    Vec3d translation = m_view_matrix.translation() + m_view_rotation * m_target;
+	    Vec3d axis        = m_view_rotation.conjugate() * rotation_rad.normalized();
+        m_view_rotation *= Eigen::Quaterniond(Eigen::AngleAxisd(angle, axis));
+        m_view_rotation.normalize();
+	    m_view_matrix.fromPositionOrientationScale(m_view_rotation * (-m_target) + translation, m_view_rotation, Vec3d(1., 1., 1.));
+	    update_zenit();
+	}
+}
+
+double Camera::min_zoom() const
+{
+    return 0.7 * calc_zoom_to_bounding_box_factor(m_scene_box);
+}
+
 std::pair<double, double> Camera::calc_tight_frustrum_zs_around(const BoundingBoxf3& box) const
 {
     std::pair<double, double> ret;
+    auto& [near_z, far_z] = ret;
 
-    while (true)
+    // box in eye space
+    BoundingBoxf3 eye_box = box.transformed(m_view_matrix);
+    near_z = -eye_box.max(2);
+    far_z = -eye_box.min(2);
+
+    // apply margin
+    near_z -= FrustrumZMargin;
+    far_z += FrustrumZMargin;
+
+    // ensure min size
+    if (far_z - near_z < FrustrumMinZRange)
     {
-        ret = std::make_pair(DBL_MAX, -DBL_MAX);
+        double mid_z = 0.5 * (near_z + far_z);
+        double half_size = 0.5 * FrustrumMinZRange;
+        near_z = mid_z - half_size;
+        far_z = mid_z + half_size;
+    }
 
-        // box vertices in world space
-        std::vector<Vec3d> vertices;
-        vertices.reserve(8);
-        vertices.push_back(box.min);
-        vertices.emplace_back(box.max(0), box.min(1), box.min(2));
-        vertices.emplace_back(box.max(0), box.max(1), box.min(2));
-        vertices.emplace_back(box.min(0), box.max(1), box.min(2));
-        vertices.emplace_back(box.min(0), box.min(1), box.max(2));
-        vertices.emplace_back(box.max(0), box.min(1), box.max(2));
-        vertices.push_back(box.max);
-        vertices.emplace_back(box.min(0), box.max(1), box.max(2));
-
-        // set the Z range in eye coordinates (negative Zs are in front of the camera)
-        for (const Vec3d& v : vertices)
-        {
-            double z = -(m_view_matrix * v)(2);
-            ret.first = std::min(ret.first, z);
-            ret.second = std::max(ret.second, z);
-        }
-
-        // apply margin
-        ret.first -= FrustrumZMargin;
-        ret.second += FrustrumZMargin;
-
-        // ensure min size
-        if (ret.second - ret.first < FrustrumMinZRange)
-        {
-            double mid_z = 0.5 * (ret.first + ret.second);
-            double half_size = 0.5 * FrustrumMinZRange;
-            ret.first = mid_z - half_size;
-            ret.second = mid_z + half_size;
-        }
-
-        if (ret.first >= FrustrumMinNearZ)
-            break;
-
-        // ensure min Near Z
-        set_distance(m_distance + FrustrumMinNearZ - ret.first);
+    if (near_z < FrustrumMinNearZ)
+    {
+        float delta = FrustrumMinNearZ - near_z;
+        set_distance(m_distance + delta);
+        near_z += delta;
+        far_z += delta;
+    }
+    else if ((near_z > 2.0 * FrustrumMinNearZ) && (m_distance > DefaultDistance))
+    {
+        float delta = m_distance - DefaultDistance;
+        set_distance(DefaultDistance);
+        near_z -= delta;
+        far_z -= delta;
     }
 
     return ret;
 }
 
 #if ENABLE_THUMBNAIL_GENERATOR
-double Camera::calc_zoom_to_bounding_box_factor(const BoundingBoxf3& box, int canvas_w, int canvas_h, double margin_factor) const
+double Camera::calc_zoom_to_bounding_box_factor(const BoundingBoxf3& box, double margin_factor) const
 #else
 double Camera::calc_zoom_to_bounding_box_factor(const BoundingBoxf3& box, int canvas_w, int canvas_h) const
 #endif // ENABLE_THUMBNAIL_GENERATOR
@@ -419,9 +400,6 @@ double Camera::calc_zoom_to_bounding_box_factor(const BoundingBoxf3& box, int ca
     // project the box vertices on a plane perpendicular to the camera forward axis
     // then calculates the vertices coordinate on this plane along the camera xy axes
 
-    // ensure that the view matrix is updated
-    apply_view_matrix();
-
     Vec3d right = get_dir_right();
     Vec3d up = get_dir_up();
     Vec3d forward = get_dir_forward();
@@ -477,11 +455,11 @@ double Camera::calc_zoom_to_bounding_box_factor(const BoundingBoxf3& box, int ca
     dx *= margin_factor;
     dy *= margin_factor;
 
-    return std::min((double)canvas_w / dx, (double)canvas_h / dy);
+    return std::min((double)m_viewport[2] / dx, (double)m_viewport[3] / dy);
 }
 
 #if ENABLE_THUMBNAIL_GENERATOR
-double Camera::calc_zoom_to_volumes_factor(const GLVolumePtrs& volumes, int canvas_w, int canvas_h, Vec3d& center, double margin_factor) const
+double Camera::calc_zoom_to_volumes_factor(const GLVolumePtrs& volumes, Vec3d& center, double margin_factor) const
 {
     if (volumes.empty())
         return -1.0;
@@ -489,9 +467,6 @@ double Camera::calc_zoom_to_volumes_factor(const GLVolumePtrs& volumes, int canv
     // project the volumes vertices on a plane perpendicular to the camera forward axis
     // then calculates the vertices coordinate on this plane along the camera xy axes
 
-    // ensure that the view matrix is updated
-    apply_view_matrix();
-
     Vec3d right = get_dir_right();
     Vec3d up = get_dir_up();
     Vec3d forward = get_dir_forward();
@@ -542,14 +517,85 @@ double Camera::calc_zoom_to_volumes_factor(const GLVolumePtrs& volumes, int canv
     if ((dx <= 0.0) || (dy <= 0.0))
         return -1.0f;
 
-    return std::min((double)canvas_w / dx, (double)canvas_h / dy);
+    return std::min((double)m_viewport[2] / dx, (double)m_viewport[3] / dy);
 }
 #endif // ENABLE_THUMBNAIL_GENERATOR
 
 void Camera::set_distance(double distance) const
 {
-    m_distance = distance;
-    apply_view_matrix();
+    if (m_distance != distance)
+    {
+        m_view_matrix.translate((distance - m_distance) * get_dir_forward());
+        m_distance = distance;
+    }
+}
+
+void Camera::look_at(const Vec3d& position, const Vec3d& target, const Vec3d& up)
+{
+    Vec3d unit_z = (position - target).normalized();
+    Vec3d unit_x = up.cross(unit_z).normalized();
+    Vec3d unit_y = unit_z.cross(unit_x).normalized();
+
+    m_target = target;
+    m_distance = (position - target).norm();
+    Vec3d new_position = m_target + m_distance * unit_z;
+
+    m_view_matrix(0, 0) = unit_x(0);
+    m_view_matrix(0, 1) = unit_x(1);
+    m_view_matrix(0, 2) = unit_x(2);
+    m_view_matrix(0, 3) = -unit_x.dot(new_position);
+
+    m_view_matrix(1, 0) = unit_y(0);
+    m_view_matrix(1, 1) = unit_y(1);
+    m_view_matrix(1, 2) = unit_y(2);
+    m_view_matrix(1, 3) = -unit_y.dot(new_position);
+
+    m_view_matrix(2, 0) = unit_z(0);
+    m_view_matrix(2, 1) = unit_z(1);
+    m_view_matrix(2, 2) = unit_z(2);
+    m_view_matrix(2, 3) = -unit_z.dot(new_position);
+
+    m_view_matrix(3, 0) = 0.0;
+    m_view_matrix(3, 1) = 0.0;
+    m_view_matrix(3, 2) = 0.0;
+    m_view_matrix(3, 3) = 1.0;
+
+    // Initialize the rotation quaternion from the rotation submatrix of of m_view_matrix.
+    m_view_rotation = Eigen::Quaterniond(m_view_matrix.matrix().template block<3, 3>(0, 0));
+    m_view_rotation.normalize();
+
+    update_zenit();
+}
+
+void Camera::set_default_orientation()
+{
+    m_zenit = 45.0f;
+    double theta_rad = Geometry::deg2rad(-(double)m_zenit);
+    double phi_rad = Geometry::deg2rad(45.0);
+    double sin_theta = ::sin(theta_rad);
+    Vec3d camera_pos = m_target + m_distance * Vec3d(sin_theta * ::sin(phi_rad), sin_theta * ::cos(phi_rad), ::cos(theta_rad));
+    m_view_rotation = Eigen::AngleAxisd(theta_rad, Vec3d::UnitX()) * Eigen::AngleAxisd(phi_rad, Vec3d::UnitZ());
+    m_view_rotation.normalize();
+    m_view_matrix.fromPositionOrientationScale(m_view_rotation * (- camera_pos), m_view_rotation, Vec3d(1., 1., 1.));
+}
+
+Vec3d Camera::validate_target(const Vec3d& target) const
+{
+    BoundingBoxf3 test_box = m_scene_box;
+    test_box.translate(-m_scene_box.center());
+    // We may let this factor be customizable
+    static const double ScaleFactor = 1.5;
+    test_box.scale(ScaleFactor);
+    test_box.translate(m_scene_box.center());
+
+    return Vec3d(std::clamp(target(0), test_box.min(0), test_box.max(0)),
+        std::clamp(target(1), test_box.min(1), test_box.max(1)),
+        std::clamp(target(2), test_box.min(2), test_box.max(2)));
+}
+
+void Camera::update_zenit()
+{
+    m_zenit = Geometry::rad2deg(0.5 * M_PI - std::acos(std::clamp(-get_dir_forward().dot(Vec3d::UnitZ()), -1.0, 1.0)));
 }
 
 } // GUI
