@@ -86,7 +86,31 @@ bool GLToolbarItem::update_enabled_state()
 
 void GLToolbarItem::render(unsigned int tex_id, float left, float right, float bottom, float top, unsigned int tex_width, unsigned int tex_height, unsigned int icon_size) const
 {
-    GLTexture::render_sub_texture(tex_id, left, right, bottom, top, get_uvs(tex_width, tex_height, icon_size));
+    auto uvs = [this](unsigned int tex_width, unsigned int tex_height, unsigned int icon_size) ->GLTexture::Quad_UVs
+    {
+        assert((tex_width != 0) && (tex_height != 0));
+        GLTexture::Quad_UVs ret;
+        // tiles in the texture are spaced by 1 pixel
+        float icon_size_px = (float)(tex_width - 1) / (float)Num_States;
+        float inv_tex_width = 1.0f / (float)tex_width;
+        float inv_tex_height = 1.0f / (float)tex_height;
+        // tiles in the texture are spaced by 1 pixel
+        float u_offset = 1.0f * inv_tex_width;
+        float v_offset = 1.0f * inv_tex_height;
+        float du = icon_size_px * inv_tex_width;
+        float dv = icon_size_px * inv_tex_height;
+        float left = u_offset + (float)m_state * du;
+        float right = left + du - u_offset;
+        float top = v_offset + (float)m_data.sprite_id * dv;
+        float bottom = top + dv - v_offset;
+        ret.left_top = { left, top };
+        ret.left_bottom = { left, bottom };
+        ret.right_bottom = { right, bottom };
+        ret.right_top = { right, top };
+        return ret;
+    };
+
+    GLTexture::render_sub_texture(tex_id, left, right, bottom, top, uvs(tex_width, tex_height, icon_size));
 
     if (is_pressed())
     {
@@ -97,28 +121,6 @@ void GLToolbarItem::render(unsigned int tex_id, float left, float right, float b
     }
 }
 
-GLTexture::Quad_UVs GLToolbarItem::get_uvs(unsigned int tex_width, unsigned int tex_height, unsigned int icon_size) const
-{
-    GLTexture::Quad_UVs uvs;
-
-    float inv_tex_width = (tex_width != 0) ? 1.0f / (float)tex_width : 0.0f;
-    float inv_tex_height = (tex_height != 0) ? 1.0f / (float)tex_height : 0.0f;
-
-    float scaled_icon_width = (float)icon_size * inv_tex_width;
-    float scaled_icon_height = (float)icon_size * inv_tex_height;
-    float left = (float)m_state * scaled_icon_width;
-    float right = left + scaled_icon_width;
-    float top = (float)m_data.sprite_id * scaled_icon_height;
-    float bottom = top + scaled_icon_height;
-
-    uvs.left_top = { left, top };
-    uvs.left_bottom = { left, bottom };
-    uvs.right_bottom = { right, bottom };
-    uvs.right_top = { right, top };
-    
-    return uvs;
-}
-
 BackgroundTexture::Metadata::Metadata()
     : filename("")
     , left(0)
@@ -345,9 +347,9 @@ bool GLToolbar::is_any_item_pressed() const
     return false;
 }
 
-unsigned int GLToolbar::get_item_id(const std::string& name) const
+int GLToolbar::get_item_id(const std::string& name) const
 {
-    for (unsigned int i = 0; i < (unsigned int)m_items.size(); ++i)
+    for (int i = 0; i < (int)m_items.size(); ++i)
     {
         if (m_items[i]->get_name() == name)
             return i;
@@ -356,19 +358,9 @@ unsigned int GLToolbar::get_item_id(const std::string& name) const
     return -1;
 }
 
-void GLToolbar::force_left_action(unsigned int item_id, GLCanvas3D& parent)
+void GLToolbar::get_additional_tooltip(int item_id, std::string& text)
 {
-    do_action(GLToolbarItem::Left, item_id, parent, false);
-}
-
-void GLToolbar::force_right_action(unsigned int item_id, GLCanvas3D& parent)
-{
-    do_action(GLToolbarItem::Right, item_id, parent, false);
-}
-
-void GLToolbar::get_additional_tooltip(unsigned int item_id, std::string& text)
-{
-    if (item_id < (unsigned int)m_items.size())
+    if ((0 <= item_id) && (item_id < (int)m_items.size()))
     {
         GLToolbarItem* item = m_items[item_id];
         if (item != nullptr)
@@ -378,12 +370,12 @@ void GLToolbar::get_additional_tooltip(unsigned int item_id, std::string& text)
         }
     }
 
-    text = L("");
+    text.clear();
 }
 
-void GLToolbar::set_additional_tooltip(unsigned int item_id, const std::string& text)
+void GLToolbar::set_additional_tooltip(int item_id, const std::string& text)
 {
-    if (item_id < (unsigned int)m_items.size())
+    if ((0 <= item_id) && (item_id < (int)m_items.size()))
     {
         GLToolbarItem* item = m_items[item_id];
         if (item != nullptr)
@@ -429,14 +421,60 @@ bool GLToolbar::on_mouse(wxMouseEvent& evt, GLCanvas3D& parent)
     // mouse anywhere
     if (!evt.Dragging() && !evt.Leaving() && !evt.Entering() && (m_mouse_capture.parent != nullptr))
     {
-        if (m_mouse_capture.any() && (evt.LeftUp() || evt.MiddleUp() || evt.RightUp()))
+        if (m_mouse_capture.any() && (evt.LeftUp() || evt.MiddleUp() || evt.RightUp())) {
             // prevents loosing selection into the scene if mouse down was done inside the toolbar and mouse up was down outside it,
             // as when switching between views
-            processed = true;
-
+            m_mouse_capture.reset();
+            if (contains_mouse(mouse_pos, parent) == -1)
+                // mouse is outside the toolbar
+                m_tooltip.clear();
+            return true;
+        }
         m_mouse_capture.reset();
     }
 
+#if ENABLE_MODIFIED_TOOLBAR_MOUSE_EVENT_HANDLING
+    if (evt.Moving())
+        m_tooltip = update_hover_state(mouse_pos, parent);
+    else if (evt.LeftUp())
+    {
+        if (m_mouse_capture.left)
+        {
+            processed = true;
+            m_mouse_capture.left = false;
+        }
+        else
+            return false;
+    }
+    else if (evt.MiddleUp())
+    {
+        if (m_mouse_capture.middle)
+        {
+            processed = true;
+            m_mouse_capture.middle = false;
+        }
+        else
+            return false;
+    }
+    else if (evt.RightUp())
+    {
+        if (m_mouse_capture.right)
+        {
+            processed = true;
+            m_mouse_capture.right = false;
+        }
+        else
+            return false;
+    }
+    else if (evt.Dragging())
+    {
+        if (m_mouse_capture.any())
+            // if the button down was done on this toolbar, prevent from dragging into the scene
+            processed = true;
+        else
+            return false;
+    }
+#else
     if (evt.Moving())
         m_tooltip = update_hover_state(mouse_pos, parent);
     else if (evt.LeftUp())
@@ -448,12 +486,13 @@ bool GLToolbar::on_mouse(wxMouseEvent& evt, GLCanvas3D& parent)
     else if (evt.Dragging() && m_mouse_capture.any())
         // if the button down was done on this toolbar, prevent from dragging into the scene
         processed = true;
+#endif // ENABLE_MODIFIED_TOOLBAR_MOUSE_EVENT_HANDLING
 
     int item_id = contains_mouse(mouse_pos, parent);
     if (item_id == -1)
     {
         // mouse is outside the toolbar
-        m_tooltip = L("");
+        m_tooltip.clear();
     }
     else
     {
@@ -466,7 +505,7 @@ bool GLToolbar::on_mouse(wxMouseEvent& evt, GLCanvas3D& parent)
             if ((item_id != -2) && !m_items[item_id]->is_separator() && ((m_pressed_toggable_id == -1) || (m_items[item_id]->get_last_action_type() == GLToolbarItem::Left)))
             {
                 // mouse is inside an icon
-                do_action(GLToolbarItem::Left, (unsigned int)item_id, parent, true);
+                do_action(GLToolbarItem::Left, item_id, parent, true);
                 parent.set_as_dirty();
             }
         }
@@ -483,12 +522,14 @@ bool GLToolbar::on_mouse(wxMouseEvent& evt, GLCanvas3D& parent)
             if ((item_id != -2) && !m_items[item_id]->is_separator() && ((m_pressed_toggable_id == -1) || (m_items[item_id]->get_last_action_type() == GLToolbarItem::Right)))
             {
                 // mouse is inside an icon
-                do_action(GLToolbarItem::Right, (unsigned int)item_id, parent, true);
+                do_action(GLToolbarItem::Right, item_id, parent, true);
                 parent.set_as_dirty();
             }
         }
+#if !ENABLE_MODIFIED_TOOLBAR_MOUSE_EVENT_HANDLING
         else if (evt.LeftUp())
             processed = true;
+#endif // !ENABLE_MODIFIED_TOOLBAR_MOUSE_EVENT_HANDLING
     }
 
     return processed;
@@ -556,11 +597,11 @@ float GLToolbar::get_main_size() const
     return size * m_layout.scale;
 }
 
-void GLToolbar::do_action(GLToolbarItem::EActionType type, unsigned int item_id, GLCanvas3D& parent, bool check_hover)
+void GLToolbar::do_action(GLToolbarItem::EActionType type, int item_id, GLCanvas3D& parent, bool check_hover)
 {
     if ((m_pressed_toggable_id == -1) || (m_pressed_toggable_id == item_id))
     {
-        if (item_id < (unsigned int)m_items.size())
+        if ((0 <= item_id) && (item_id < (int)m_items.size()))
         {
             GLToolbarItem* item = m_items[item_id];
             if ((item != nullptr) && !item->is_separator() && (!check_hover || item->is_hovered()))
@@ -620,7 +661,7 @@ void GLToolbar::do_action(GLToolbarItem::EActionType type, unsigned int item_id,
 std::string GLToolbar::update_hover_state(const Vec2d& mouse_pos, GLCanvas3D& parent)
 {
     if (!m_enabled)
-        return L("");
+        return "";
 
     switch (m_layout.type)
     {
@@ -634,8 +675,7 @@ std::string GLToolbar::update_hover_state_horizontal(const Vec2d& mouse_pos, GLC
 {
     // NB: mouse_pos is already scaled appropriately
 
-    float zoom = (float)parent.get_camera().get_zoom();
-    float inv_zoom = (zoom != 0.0f) ? 1.0f / zoom : 0.0f;
+    float inv_zoom = (float)parent.get_camera().get_inv_zoom();
     float factor = m_layout.scale * inv_zoom;
 
     Size cnv_size = parent.get_canvas_size();
@@ -675,7 +715,7 @@ std::string GLToolbar::update_hover_state_horizontal(const Vec2d& mouse_pos, GLC
                 {
                     const std::string& additional_tooltip = item->get_additional_tooltip();
                     if (!additional_tooltip.empty())
-                        tooltip += L("\n") + additional_tooltip;
+                        tooltip += "\n" + additional_tooltip;
                 }
             }
 
@@ -739,8 +779,7 @@ std::string GLToolbar::update_hover_state_vertical(const Vec2d& mouse_pos, GLCan
 {
     // NB: mouse_pos is already scaled appropriately
 
-    float zoom = (float)parent.get_camera().get_zoom();
-    float inv_zoom = (zoom != 0.0f) ? 1.0f / zoom : 0.0f;
+    float inv_zoom = (float)parent.get_camera().get_inv_zoom();
     float factor = m_layout.scale * inv_zoom;
 
     Size cnv_size = parent.get_canvas_size();
@@ -779,7 +818,7 @@ std::string GLToolbar::update_hover_state_vertical(const Vec2d& mouse_pos, GLCan
                 {
                     const std::string& additional_tooltip = item->get_additional_tooltip();
                     if (!additional_tooltip.empty())
-                        tooltip += L("\n") + additional_tooltip;
+                        tooltip += "\n" + additional_tooltip;
                 }
             }
 
@@ -856,8 +895,7 @@ int GLToolbar::contains_mouse_horizontal(const Vec2d& mouse_pos, const GLCanvas3
 {
     // NB: mouse_pos is already scaled appropriately
 
-    float zoom = (float)parent.get_camera().get_zoom();
-    float inv_zoom = (zoom != 0.0f) ? 1.0f / zoom : 0.0f;
+    float inv_zoom = (float)parent.get_camera().get_inv_zoom();
     float factor = m_layout.scale * inv_zoom;
 
     Size cnv_size = parent.get_canvas_size();
@@ -871,11 +909,10 @@ int GLToolbar::contains_mouse_horizontal(const Vec2d& mouse_pos, const GLCanvas3
     float left = m_layout.left + scaled_border;
     float top = m_layout.top - scaled_border;
 
-    int id = -1;
-    
-    for (GLToolbarItem* item : m_items)
+
+    for (size_t id=0; id<m_items.size(); ++id)
     {
-        ++id;
+        GLToolbarItem* item = m_items[id];
         
         if (!item->is_visible())
             continue;
@@ -931,8 +968,7 @@ int GLToolbar::contains_mouse_vertical(const Vec2d& mouse_pos, const GLCanvas3D&
 {
     // NB: mouse_pos is already scaled appropriately
 
-    float zoom = (float)parent.get_camera().get_zoom();
-    float inv_zoom = (zoom != 0.0f) ? 1.0f / zoom : 0.0f;
+    float inv_zoom = (float)parent.get_camera().get_inv_zoom();
     float factor = m_layout.scale * inv_zoom;
 
     Size cnv_size = parent.get_canvas_size();
@@ -946,11 +982,9 @@ int GLToolbar::contains_mouse_vertical(const Vec2d& mouse_pos, const GLCanvas3D&
     float left = m_layout.left + scaled_border;
     float top = m_layout.top - scaled_border;
 
-    int id = -1;
-
-    for (GLToolbarItem* item : m_items)
+    for (size_t id=0; id<m_items.size(); ++id)
     {
-        ++id;
+        GLToolbarItem* item = m_items[id];
 
         if (!item->is_visible())
             continue;
@@ -1086,8 +1120,7 @@ void GLToolbar::render_horizontal(const GLCanvas3D& parent) const
     int tex_width = m_icons_texture.get_width();
     int tex_height = m_icons_texture.get_height();
 
-    float zoom = (float)parent.get_camera().get_zoom();
-    float inv_zoom = (zoom != 0.0f) ? 1.0f / zoom : 0.0f;
+    float inv_zoom = (float)parent.get_camera().get_inv_zoom();
     float factor = inv_zoom * m_layout.scale;
 
     float scaled_icons_size = m_layout.icons_size * factor;
@@ -1135,8 +1168,7 @@ void GLToolbar::render_vertical(const GLCanvas3D& parent) const
     int tex_width = m_icons_texture.get_width();
     int tex_height = m_icons_texture.get_height();
 
-    float zoom = (float)parent.get_camera().get_zoom();
-    float inv_zoom = (zoom != 0.0f) ? 1.0f / zoom : 0.0f;
+    float inv_zoom = (float)parent.get_camera().get_inv_zoom();
     float factor = inv_zoom * m_layout.scale;
 
     float scaled_icons_size = m_layout.icons_size * factor;
@@ -1251,7 +1283,7 @@ bool GLToolbar::update_items_enabled_state()
 {
     bool ret = false;
 
-    for (unsigned int i = 0; i < (unsigned int)m_items.size(); ++i)
+    for (int i = 0; i < (int)m_items.size(); ++i)
     {
         GLToolbarItem* item = m_items[i];
         ret |= item->update_enabled_state();
