@@ -2,6 +2,8 @@
 #define slic3r_Preset_hpp_
 
 #include <deque>
+#include <set>
+#include <unordered_map>
 
 #include <boost/filesystem/path.hpp>
 #include <boost/property_tree/ptree_fwd.hpp>
@@ -59,6 +61,10 @@ public:
         PrinterTechnology           technology;
         std::string                 family;
         std::vector<PrinterVariant> variants;
+        std::vector<std::string>	default_materials;
+        // Vendor & Printer Model specific print bed model & texture.
+        std::string 			 	bed_model;
+        std::string 				bed_texture;
 
         PrinterVariant*       variant(const std::string &name) {
             for (auto &v : this->variants)
@@ -71,11 +77,16 @@ public:
     };
     std::vector<PrinterModel>          models;
 
+    std::set<std::string>              default_filaments;
+    std::set<std::string>              default_sla_materials;
+
     VendorProfile() {}
     VendorProfile(std::string id) : id(std::move(id)) {}
 
     bool 		valid() const { return ! name.empty() && ! id.empty() && config_version.valid(); }
 
+    // Load VendorProfile from an ini file.
+    // If `load_all` is false, only the header with basic info (name, version, URLs) is loaded.
     static VendorProfile from_ini(const boost::filesystem::path &path, bool load_all=true);
     static VendorProfile from_ini(const boost::property_tree::ptree &tree, const boost::filesystem::path &path, bool load_all=true);
 
@@ -86,6 +97,23 @@ public:
     bool        operator==(const VendorProfile &rhs) const { return this->id == rhs.id; }
 };
 
+class Preset;
+
+// Helper to hold a profile with its vendor definition, where the vendor definition may have been extracted from a parent system preset.
+// The parent preset is only accessible through PresetCollection, therefore to allow definition of the various is_compatible_with methods
+// outside of the PresetCollection, this composite is returned by PresetCollection::get_preset_with_vendor_profile() when needed.
+struct PresetWithVendorProfile {
+	PresetWithVendorProfile(const Preset &preset, const VendorProfile *vendor) : preset(preset), vendor(vendor) {}
+	const Preset 		&preset;
+	const VendorProfile *vendor;
+};
+
+// Note: it is imporant that map is used here rather than unordered_map,
+// because we need iterators to not be invalidated,
+// because Preset and the ConfigWizard hold pointers to VendorProfiles.
+// XXX: maybe set is enough (cf. changes in Wizard)
+typedef std::map<std::string, VendorProfile> VendorMap;
+
 class Preset
 {
 public:
@@ -137,6 +165,13 @@ public:
     // Configuration data, loaded from a file, or set from the defaults.
     DynamicPrintConfig  config;
 
+    // Alias of the preset
+    std::string         alias;
+    // List of profile names, from which this profile was renamed at some point of time.
+    // This list is then used to match profiles by their names when loaded from .gcode, .3mf, .amf,
+    // and to match the "inherits" field of user profiles with updated system profiles.
+    std::vector<std::string> renamed_from;
+
     void                save();
 
     // Return a label of this preset, consisting of a name and a "(modified)" suffix, if this preset is dirty.
@@ -147,10 +182,6 @@ public:
     void                set_dirty(bool dirty = true) { this->is_dirty = dirty; }
     void                reset_dirty() { this->is_dirty = false; }
 
-    bool                is_compatible_with_print(const Preset &active_print) const;
-    bool                is_compatible_with_printer(const Preset &active_printer, const DynamicPrintConfig *extra_config) const;
-    bool                is_compatible_with_printer(const Preset &active_printer) const;
-
     // Returns the name of the preset, from which this preset inherits.
     static std::string& inherits(DynamicPrintConfig &cfg) { return cfg.option<ConfigOptionString>("inherits", true)->value; }
     std::string&        inherits() { return Preset::inherits(this->config); }
@@ -184,14 +215,11 @@ public:
     // This call returns a reference, it may add a new entry into the DynamicPrintConfig.
     PrinterTechnology&  printer_technology_ref() { return this->config.option<ConfigOptionEnum<PrinterTechnology>>("printer_technology", true)->value; }
 
-    // Mark this preset as compatible if it is compatible with active_printer.
-    bool                update_compatible(const Preset &active_printer, const DynamicPrintConfig *extra_config, const Preset *active_print = nullptr);
-
     // Set is_visible according to application config
     void                set_visible_from_appconfig(const AppConfig &app_config);
 
     // Resize the extruder specific fields, initialize them with the content of the 1st extruder.
-    void                set_num_extruders(unsigned int n) { set_num_extruders(this->config, n); }
+    void                set_num_extruders(unsigned int n) { this->config.set_num_extruders(n); }
 
     // Sort lexicographically by a preset name. The preset name shall be unique across a single PresetCollection.
     bool                operator<(const Preset &other) const { return this->name < other.name; }
@@ -209,6 +237,7 @@ public:
 
 	static void                             update_suffix_modified();
     static const std::string&               suffix_modified();
+    static std::string                      remove_suffix_modified(const std::string& name);
     static void                             normalize(DynamicPrintConfig &config);
     // Report configuration fields, which are misplaced into a wrong group, remove them from the config.
     static std::string                      remove_invalid_keys(DynamicPrintConfig &config, const DynamicPrintConfig &default_config);
@@ -216,9 +245,19 @@ public:
 protected:
     friend class        PresetCollection;
     friend class        PresetBundle;
-    // Resize the extruder specific vectors ()
-    static void         set_num_extruders(DynamicPrintConfig &config, unsigned int n);
-    static std::string  remove_suffix_modified(const std::string &name);
+};
+
+bool is_compatible_with_print  (const PresetWithVendorProfile &preset, const PresetWithVendorProfile &active_print, const PresetWithVendorProfile &active_printer);
+bool is_compatible_with_printer(const PresetWithVendorProfile &preset, const PresetWithVendorProfile &active_printer, const DynamicPrintConfig *extra_config);
+bool is_compatible_with_printer(const PresetWithVendorProfile &preset, const PresetWithVendorProfile &active_printer);
+
+enum class PresetSelectCompatibleType {
+	// Never select a compatible preset if the newly selected profile is not compatible.
+	Never,
+	// Only select a compatible preset if the active profile used to be compatible, but it is no more.
+	OnlyIfWasCompatible,
+	// Always select a compatible preset if the active profile is no more compatible.
+	Always
 };
 
 // Collections of presets of the same type (one of the Print, Filament or Printer type).
@@ -273,17 +312,20 @@ public:
     // Save the preset under a new name. If the name is different from the old one,
     // a new preset is stored into the list of presets.
     // All presets are marked as not modified and the new preset is activated.
-    void            save_current_preset(const std::string &new_name);
+    void            save_current_preset(const std::string &new_name, bool detach = false);
 
     // Delete the current preset, activate the first visible preset.
     // returns true if the preset was deleted successfully.
     bool            delete_current_preset();
+    // Delete the current preset, activate the first visible preset.
+    // returns true if the preset was deleted successfully.
+    bool            delete_preset(const std::string& name);
 
     // Load default bitmap to be placed at the wxBitmapComboBox of a MainFrame.
-    void            load_bitmap_default(wxWindow *window, const std::string &file_name);
+    void            load_bitmap_default(const std::string &file_name);
 
     // Load "add new printer" bitmap to be placed at the wxBitmapComboBox of a MainFrame.
-    void            load_bitmap_add(wxWindow *window, const std::string &file_name);
+    void            load_bitmap_add(const std::string &file_name);
 
     // Compatible & incompatible marks, to be placed at the wxBitmapComboBox items.
     void            set_bitmap_compatible  (const wxBitmap *bmp) { m_bitmap_compatible   = bmp; }
@@ -308,15 +350,23 @@ public:
     // The parent preset may be a system preset or a user preset, which will be
     // reflected by the UI.
     const Preset*   get_selected_preset_parent() const;
-	// get parent preset for some child preset
+	// Get parent preset for a child preset, based on the "inherits" field of a child,
+	// where the "inherits" profile name is searched for in both m_presets and m_map_system_profile_renamed.
 	const Preset*	get_preset_parent(const Preset& child) const;
 	// Return the selected preset including the user modifications.
     Preset&         get_edited_preset()         { return m_edited_preset; }
     const Preset&   get_edited_preset() const   { return m_edited_preset; }
 
+    // Return vendor of the first parent profile, for which the vendor is defined, or null if such profile does not exist.
+    PresetWithVendorProfile get_preset_with_vendor_profile(const Preset &preset) const;
+    PresetWithVendorProfile get_edited_preset_with_vendor_profile() const { return this->get_preset_with_vendor_profile(this->get_edited_preset()); }
+
+    const std::string& 		get_preset_name_by_alias(const std::string& alias) const;
+	const std::string*		get_preset_name_renamed(const std::string &old_name) const;
+
 	// used to update preset_choice from Tab
 	const std::deque<Preset>&	get_presets() const	{ return m_presets; }
-	int						get_idx_selected()	{ return m_idx_selected; }
+	int							get_idx_selected()	{ return m_idx_selected; }
 	static const std::string&	get_suffix_modified();
 
     // Return a preset possibly with modifications.
@@ -344,7 +394,8 @@ public:
         size_t n = this->m_presets.size();
         size_t i_compatible = n;
         for (; i < n; ++ i)
-            if (m_presets[i].is_compatible) {
+            // Since we use the filament selection from Wizard, it's needed to control the preset visibility too 
+            if (m_presets[i].is_compatible && m_presets[i].is_visible) {
                 if (prefered_condition(m_presets[i].name))
                     return i;
                 if (i_compatible == n)
@@ -371,13 +422,13 @@ public:
 
     // For Print / Filament presets, disable those, which are not compatible with the printer.
     template<typename PreferedCondition>
-    void            update_compatible(const Preset &active_printer, const Preset *active_print, bool select_other_if_incompatible, PreferedCondition prefered_condition)
+    void            update_compatible(const PresetWithVendorProfile &active_printer, const PresetWithVendorProfile *active_print, PresetSelectCompatibleType select_other_if_incompatible, PreferedCondition prefered_condition)
     {
         if (this->update_compatible_internal(active_printer, active_print, select_other_if_incompatible) == (size_t)-1)
             // Find some other compatible preset, or the "-- default --" preset.
             this->select_preset(this->first_compatible_idx(prefered_condition));        
     }
-    void            update_compatible(const Preset &active_printer, const Preset *active_print, bool select_other_if_incompatible)
+    void            update_compatible(const PresetWithVendorProfile &active_printer, const PresetWithVendorProfile *active_print, PresetSelectCompatibleType select_other_if_incompatible)
         { this->update_compatible(active_printer, active_print, select_other_if_incompatible, [](const std::string&){return true;}); }
 
     size_t          num_visible() const { return std::count_if(m_presets.begin(), m_presets.end(), [](const Preset &preset){return preset.is_visible;}); }
@@ -401,7 +452,7 @@ public:
     // Update the choice UI from the list of presets.
     // Only the compatible presets are shown.
     // If an incompatible preset is selected, it is shown as well.
-    void            update_platter_ui(GUI::PresetComboBox *ui);
+    void            update_plater_ui(GUI::PresetComboBox *ui);
 
     // Update a dirty floag of the current preset, update the labels of the UI component accordingly.
     // Return true if the dirty flag changed.
@@ -432,7 +483,13 @@ protected:
     bool            select_preset_by_name_strict(const std::string &name);
 
     // Merge one vendor's presets with the other vendor's presets, report duplicates.
-    std::vector<std::string> merge_presets(PresetCollection &&other, const std::set<VendorProfile> &new_vendors);
+    std::vector<std::string> merge_presets(PresetCollection &&other, const VendorMap &new_vendors);
+
+    // Update m_map_alias_to_profile_name from loaded system profiles.
+	void 			update_map_alias_to_profile_name();
+
+    // Update m_map_system_profile_renamed from loaded system profiles.
+    void 			update_map_system_profile_renamed();
 
 private:
     PresetCollection();
@@ -459,8 +516,16 @@ private:
     }
     std::deque<Preset>::const_iterator find_preset_internal(const std::string &name) const
         { return const_cast<PresetCollection*>(this)->find_preset_internal(name); }
+    std::deque<Preset>::iterator 	   find_preset_renamed(const std::string &name) {
+    	auto it_renamed = m_map_system_profile_renamed.find(name);
+    	auto it = (it_renamed == m_map_system_profile_renamed.end()) ? m_presets.end() : this->find_preset_internal(it_renamed->second);
+    	assert((it_renamed == m_map_system_profile_renamed.end()) || (it != m_presets.end() && it->name == it_renamed->second));
+    	return it;
+    }
+    std::deque<Preset>::const_iterator find_preset_renamed(const std::string &name) const
+        { return const_cast<PresetCollection*>(this)->find_preset_renamed(name); }
 
-    size_t update_compatible_internal(const Preset &active_printer, const Preset *active_print, bool unselect_if_incompatible);
+    size_t update_compatible_internal(const PresetWithVendorProfile &active_printer, const PresetWithVendorProfile *active_print, PresetSelectCompatibleType unselect_if_incompatible);
 
     static std::vector<std::string> dirty_options(const Preset *edited, const Preset *reference, const bool is_printer_type = false);
 
@@ -470,6 +535,10 @@ private:
     // Use deque to force the container to allocate an object per each entry, 
     // so that the addresses of the presets don't change during resizing of the container.
     std::deque<Preset>      m_presets;
+    // System profiles may have aliases. Map to the full profile name.
+    std::vector<std::pair<std::string, std::string>> m_map_alias_to_profile_name;
+    // Map from old system profile name to a current system profile name.
+    std::map<std::string, std::string> m_map_system_profile_renamed;
     // Initially this preset contains a copy of the selected preset. Later on, this copy may be modified by the user.
     Preset                  m_edited_preset;
     // Selected preset.
@@ -477,7 +546,7 @@ private:
     // Is the "- default -" preset suppressed?
     bool                    m_default_suppressed  = true;
     size_t                  m_num_default_presets = 0;
-    // Compatible & incompatible marks, to be placed at the wxBitmapComboBox items of a Platter.
+    // Compatible & incompatible marks, to be placed at the wxBitmapComboBox items of a Plater.
     // These bitmaps are not owned by PresetCollection, but by a PresetBundle.
     const wxBitmap         *m_bitmap_compatible   = nullptr;
     const wxBitmap         *m_bitmap_incompatible = nullptr;
@@ -510,6 +579,11 @@ public:
     const Preset*   find_by_model_id(const std::string &model_id) const;
 };
 
+namespace PresetUtils {
+	// PrinterModel of a system profile, from which this preset is derived, or null if it is not derived from a system profile.
+	const VendorProfile::PrinterModel* system_printer_model(const Preset &preset);
+} // namespace PresetUtils
+
 } // namespace Slic3r
 
 #endif /* slic3r_Preset_hpp_ */
