@@ -23,6 +23,11 @@ public:
     // Produce a textual description of a recommended thin wall thickness
     // from the provided number of perimeters and the external / internal perimeter width.
     static std::string recommended_thin_wall_thickness(const PresetBundle &preset_bundle);
+
+    // Produce a textual explanation of the combined effects of the top/bottom_solid_layers
+    // versus top/bottom_min_shell_thickness. Which of the two values wins depends
+    // on the active layer height.
+    static std::string top_bottom_shell_thickness_explanation(const PresetBundle &preset_bundle);    
 };
 
 } // namespace Slic3r
