@@ -6,10 +6,8 @@
 
 #include "PrintHost.hpp"
 
-
 namespace Slic3r {
 
-
 class DynamicPrintConfig;
 class Http;
 
@@ -17,18 +15,18 @@ class Duet : public PrintHost
 {
 public:
 	Duet(DynamicPrintConfig *config);
-	virtual ~Duet();
+	~Duet() override = default;
 
-	virtual const char* get_name() const;
+	const char* get_name() const override;
 
-	virtual bool test(wxString &curl_msg) const;
-	virtual wxString get_test_ok_msg () const;
-	virtual wxString get_test_failed_msg (wxString &msg) const;
-	virtual bool upload(PrintHostUpload upload_data, ProgressFn prorgess_fn, ErrorFn error_fn) const;
-	virtual bool has_auto_discovery() const;
-	virtual bool can_test() const;
-	virtual bool can_start_print() const;
-	virtual std::string get_host() const { return host; }
+	bool test(wxString &curl_msg) const override;
+	wxString get_test_ok_msg() const override;
+	wxString get_test_failed_msg(wxString &msg) const override;
+	bool upload(PrintHostUpload upload_data, ProgressFn prorgess_fn, ErrorFn error_fn) const override;
+	bool has_auto_discovery() const override { return false; }
+	bool can_test() const override { return true; }
+	bool can_start_print() const override { return true; }
+	std::string get_host() const override { return host; }
 
 private:
 	std::string host;
@@ -44,7 +42,6 @@ private:
 	int get_err_code_from_body(const std::string &body) const;
 };
 
-
 }
 
 #endif
