@@ -299,7 +299,7 @@ bool ConfigBase__set_deserialize(ConfigBase* THIS, const t_config_option_key &op
     size_t len;
     const char * c = SvPV(str, len);
     std::string value(c, len);
-    return THIS->set_deserialize(opt_key, value);
+    return THIS->set_deserialize_nothrow(opt_key, value);
 }
 
 void ConfigBase__set_ifndef(ConfigBase* THIS, const t_config_option_key &opt_key, SV* value, bool deserialize)
